/**
 * Legato is a configurable, lightweight web mapping client that can be
 * easily embedded into web pages and portals, CMS and individual web
 * applications. Legato is implemented in JavaScript and based on the
 * popular open source library OpenLayers.
 *
 * Copyright (C) 2010  disy Informationssysteme GmbH, http://www.disy.net
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package net.disy.commons.xml.datatype;

import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.datatype.DatatypeFactory;
import javax.xml.datatype.XMLGregorianCalendar;

import org.directwebremoting.convert.BaseV20Converter;
import org.directwebremoting.dwrp.SimpleOutboundVariable;
import org.directwebremoting.extend.Converter;
import org.directwebremoting.extend.InboundContext;
import org.directwebremoting.extend.InboundVariable;
import org.directwebremoting.extend.MarshallException;
import org.directwebremoting.extend.OutboundContext;
import org.directwebremoting.extend.OutboundVariable;
import org.directwebremoting.util.JavascriptUtil;
import org.directwebremoting.util.LocalUtil;
import org.directwebremoting.util.Logger;

public class XMLGregorianCalendarConverter extends BaseV20Converter implements Converter {

  private static final Logger log = Logger.getLogger(XMLGregorianCalendarConverter.class);

  private final DatatypeFactory datatypeFactory;

  public XMLGregorianCalendarConverter() {
    try {
      this.datatypeFactory = DatatypeFactory.newInstance();
    }
    catch (DatatypeConfigurationException dcex) {
      throw new ExceptionInInitializerError(dcex);
    }
  }

  public Object convertInbound(
      @SuppressWarnings("unchecked") Class paramType,
      InboundVariable iv,
      InboundContext inctx) {
    String string = LocalUtil.decode(iv.getValue());
    try {
      return datatypeFactory.newXMLGregorianCalendar(string);
    }
    catch (IllegalArgumentException iaex) {
      log.warn("Failed to create XMLGregorianCalendar from string [" //$NON-NLS-1$
          + string
          + "]. Returning [null]."); //$NON-NLS-1$
      return null;
    }
  }

  public OutboundVariable convertOutbound(Object data, OutboundContext outctx)
      throws MarshallException {

    if (data instanceof XMLGregorianCalendar) {
      String escaped = JavascriptUtil.escapeJavaScript(data.toString());
      return new SimpleOutboundVariable('\"' + escaped + '\"', outctx, true);
    }
    throw new MarshallException(data.getClass());
  }

}
