/**
 * Legato is a configurable, lightweight web mapping client that can be
 * easily embedded into web pages and portals, CMS and individual web
 * applications. Legato is implemented in JavaScript and based on the
 * popular open source library OpenLayers.
 *
 * Copyright (C) 2010  disy Informationssysteme GmbH, http://www.disy.net
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package net.disy.ogc.gml.v_3_1_1;

import javax.xml.namespace.QName;

import net.disy.ogc.wps.v_1_0_0.model.DataType;
import net.disy.ogc.wps.v_1_0_0.model.SchemaReference;
import net.opengis.gml.v_3_1_1.ObjectFactory;

import com.vividsolutions.jts.geom.Geometry;
import com.vividsolutions.jts.geom.GeometryCollection;
import com.vividsolutions.jts.geom.LineString;
import com.vividsolutions.jts.geom.MultiLineString;
import com.vividsolutions.jts.geom.MultiPoint;
import com.vividsolutions.jts.geom.MultiPolygon;
import com.vividsolutions.jts.geom.Point;
import com.vividsolutions.jts.geom.Polygon;

public class GmlConstants {

  private GmlConstants() {

  }

  public static final String PACKAGE_NAME = ObjectFactory.class.getPackage().getName();

  public static final String CONTEXT_PATH = GmlConstants.PACKAGE_NAME + ":" //$NON-NLS-1$
      + org.w3.smil.v_2_0.ObjectFactory.class.getPackage().getName()
      + ":" //$NON-NLS-1$
      + org.w3.smil.v_2_0.language.ObjectFactory.class.getPackage().getName();

  public static String LOCATION = "http://schemas.opengis.net/gml/3.1.1/base/gml.xsd"; //$NON-NLS-1$
  public static String NAMESPACE_URI = "http://www.opengis.net/gml"; //$NON-NLS-1$
  public static String DEFAULT_PREFIX = "gml"; //$NON-NLS-1$

  public static SchemaReference GML = new SchemaReference(LOCATION, NAMESPACE_URI, DEFAULT_PREFIX); //$NON-NLS-1$

  public final static String POINT_LOCAL_NAME = "Point"; //$NON-NLS-1$
  public final static String LINE_STRING_LOCAL_NAME = "LineString"; //$NON-NLS-1$
  public final static String POLYGON_LOCAL_NAME = "Polygon"; //$NON-NLS-1$
  public final static String GEOMETRY_LOCAL_NAME = "Geometry"; //$NON-NLS-1$

  public final static String MULTI_POINT_LOCAL_NAME = "MultiPoint"; //$NON-NLS-1$
  public final static String MULTI_LINE_STRING_LOCAL_NAME = "MultiLineString"; //$NON-NLS-1$
  public final static String MULTI_POLYGON_LOCAL_NAME = "MultiPolygon"; //$NON-NLS-1$
  public final static String MULTI_GEOMETRY_LOCAL_NAME = "MultiGeometry"; //$NON-NLS-1$

  public static String POINT_SCHEMA_DESIGNATOR = GML.elementDesignator(POINT_LOCAL_NAME);
  public static String LINE_STRING_SCHEMA_DESIGNATOR = GML
      .elementDesignator(LINE_STRING_LOCAL_NAME);
  public static String POLYGON_SCHEMA_DESIGNATOR = GML.elementDesignator(POLYGON_LOCAL_NAME);
  public static String GEOMETRY_SCHEMA_DESIGNATOR = GML.elementDesignator(GEOMETRY_LOCAL_NAME);

  public static String MULTI_POINT_SCHEMA_DESIGNATOR = GML
      .elementDesignator(MULTI_POINT_LOCAL_NAME);
  public static String MULTI_LINE_STRING_SCHEMA_DESIGNATOR = GML
      .elementDesignator(MULTI_LINE_STRING_LOCAL_NAME);
  public static String MULTI_POLYGON_SCHEMA_DESIGNATOR = GML
      .elementDesignator(MULTI_POLYGON_LOCAL_NAME);
  public static String MULTI_GEOMETRY_SCHEMA_DESIGNATOR = GML
      .elementDesignator(MULTI_GEOMETRY_LOCAL_NAME);

  public static QName POINT_ELEMENT_NAME = GML.qname(POINT_LOCAL_NAME);
  public static QName LINE_STRING_ELEMENT_NAME = GML.qname(LINE_STRING_LOCAL_NAME);
  public static QName POLYGON_ELEMENT_NAME = GML.qname(POLYGON_LOCAL_NAME);
  public static QName GEOMETRY_ELEMENT_NAME = GML.qname(GEOMETRY_LOCAL_NAME);

  public static QName MULTI_POINT_ELEMENT_NAME = GML.qname(MULTI_POINT_LOCAL_NAME);
  public static QName MULTI_LINE_STRING_ELEMENT_NAME = GML.qname(MULTI_LINE_STRING_LOCAL_NAME);
  public static QName MULTI_POLYGON_ELEMENT_NAME = GML.qname(MULTI_POLYGON_LOCAL_NAME);
  public static QName MULTI_GEOMETRY_ELEMENT_NAME = GML.qname(MULTI_GEOMETRY_LOCAL_NAME);

  public final static DataType<Polygon> POLYGON_DATA_TYPE = GML.complexDataType(
      GmlConstants.POLYGON_LOCAL_NAME,
      Polygon.class);
  public final static DataType<LineString> LINE_STRING_DATA_TYPE = GML.complexDataType(
      GmlConstants.LINE_STRING_LOCAL_NAME,
      LineString.class);
  public final static DataType<Point> POINT_DATA_TYPE = GML.complexDataType(
      GmlConstants.POINT_LOCAL_NAME,
      Point.class);
  public final static DataType<Geometry> GEOMETRY_DATA_TYPE = GML.complexDataType(
      GmlConstants.GEOMETRY_LOCAL_NAME,
      Geometry.class);
  
  public final static DataType<MultiPolygon> MULTI_POLYGON_DATA_TYPE = GML.complexDataType(
      GmlConstants.MULTI_POLYGON_LOCAL_NAME,
      MultiPolygon.class);
  public final static DataType<MultiLineString> MULTI_LINE_STRING_DATA_TYPE = GML.complexDataType(
      GmlConstants.MULTI_LINE_STRING_LOCAL_NAME,
      MultiLineString.class);
  public final static DataType<MultiPoint> MULTI_POINT_DATA_TYPE = GML.complexDataType(
      GmlConstants.MULTI_POINT_LOCAL_NAME,
      MultiPoint.class);
  public final static DataType<GeometryCollection> MULTI_GEOMETRY_DATA_TYPE = GML.complexDataType(
      GmlConstants.MULTI_GEOMETRY_LOCAL_NAME,
      GeometryCollection.class);
  
}
