/**
 * Legato is a configurable, lightweight web mapping client that can be
 * easily embedded into web pages and portals, CMS and individual web
 * applications. Legato is implemented in JavaScript and based on the
 * popular open source library OpenLayers.
 *
 * Copyright (C) 2010  disy Informationssysteme GmbH, http://www.disy.net
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package net.disy.ogc.gml.v_3_1_1.dwr;

import java.util.Map;
import java.util.TreeMap;

import net.disy.commons.dwr.convert.AbstractConverter;

import org.directwebremoting.dwrp.ObjectOutboundVariable;
import org.directwebremoting.dwrp.ParseUtil;
import org.directwebremoting.extend.InboundContext;
import org.directwebremoting.extend.InboundVariable;
import org.directwebremoting.extend.MarshallException;
import org.directwebremoting.extend.OutboundContext;
import org.directwebremoting.extend.OutboundVariable;
import org.directwebremoting.util.LocalUtil;
import org.jvnet.ogc.gml.v_3_1_1.jts.GML311ToJTSGeometryConverter;

public abstract class AbstractGeometryTypeConverter<G, C> extends AbstractConverter {

  protected abstract G createGeometry(C coordinates) throws MarshallException;

  protected abstract C createCoordinates(G geometry) throws MarshallException;

  protected abstract Class<C> getCoordinatesType();

  protected abstract String getGeometryType();

  private GML311ToJTSGeometryConverter converter = new GML311ToJTSGeometryConverter();

  protected GML311ToJTSGeometryConverter getConverter() {
    return converter;
  }

  @Override
  protected Object convertInbound(
      @SuppressWarnings({ "rawtypes" }) Class paramType,
      InboundVariable data,
      Map<String, String> tokens,
      InboundContext inctx,
      String value) throws MarshallException {

    final String coordinatesString = tokens.get("coordinates"); //$NON-NLS-1$

    final String[] split = ParseUtil.splitInbound(coordinatesString);
    final String splitValue = split[LocalUtil.INBOUND_INDEX_VALUE];
    final String splitType = split[LocalUtil.INBOUND_INDEX_TYPE];

    final InboundVariable nested = new InboundVariable(
        data.getLookup(),
        null,
        splitType,
        splitValue);

    @SuppressWarnings("unchecked")
    final C coordinates = (C) getConverterManager().convertInbound(
        getCoordinatesType(),
        nested,
        inctx,
        null);

    return createGeometry(coordinates);
  }

  @Override
  public OutboundVariable convertOutbound(Object data, OutboundContext outctx)
      throws MarshallException {

    @SuppressWarnings("unchecked")
    final G coordinateInfo = (G) data;
    final C coordinates = createCoordinates(coordinateInfo);

    //         // Where we collect out converted children
    final Map<String, OutboundVariable> ovs = new TreeMap<String, OutboundVariable>();

    // We need to do this before collecing the children to save recurrsion
    final ObjectOutboundVariable ov = new ObjectOutboundVariable(outctx);
    outctx.put(data, ov);

    ovs.put("type", getConverterManager().convertOutbound(getGeometryType(), outctx)); //$NON-NLS-1$
    ovs.put("coordinates", getConverterManager().convertOutbound(coordinates, outctx)); //$NON-NLS-1$

    ov.init(ovs, null);

    return ov;
  }
  //  @Override
  //  protected Object convertInbound(
  //      @SuppressWarnings("unchecked") Class paramType,
  //      InboundVariable data,
  //      Map<String, String> tokens,
  //      InboundContext inctx,
  //      String value) throws MarshallException {
  //
  //    final String coordinatesString = tokens.get("coordinates"); //$NON-NLS-1$
  //
  //    final String[] split = ParseUtil.splitInbound(coordinatesString);
  //    final String splitValue = split[LocalUtil.INBOUND_INDEX_VALUE];
  //    final String splitType = split[LocalUtil.INBOUND_INDEX_TYPE];
  //
  //    final InboundVariable nested = new InboundVariable(
  //        data.getLookup(),
  //        null,
  //        splitType,
  //        splitValue);
  //
  //    @SuppressWarnings("unchecked")
  //    final C coordinates = (C) getConverterManager().convertInbound(
  //        getCoordinatesType(),
  //        nested,
  //        inctx,
  //        null);
  //
  //    return createGeometry(coordinates);
  //  }

}
