/**
 * Legato is a configurable, lightweight web mapping client that can be
 * easily embedded into web pages and portals, CMS and individual web
 * applications. Legato is implemented in JavaScript and based on the
 * popular open source library OpenLayers.
 *
 * Copyright (C) 2010  disy Informationssysteme GmbH, http://www.disy.net
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package net.disy.ogc.gml.v_3_1_1.dwr;

import java.util.Map;

import net.disy.commons.dwr.convert.AbstractConverter;
import net.opengis.gml.v_3_1_1.LineStringType;
import net.opengis.gml.v_3_1_1.PointType;
import net.opengis.gml.v_3_1_1.PolygonType;

import org.directwebremoting.dwrp.ParseUtil;
import org.directwebremoting.extend.InboundContext;
import org.directwebremoting.extend.InboundVariable;
import org.directwebremoting.extend.MarshallException;
import org.directwebremoting.util.LocalUtil;

public class GeometryConverter extends AbstractConverter {

  @Override
  protected Object convertInbound(
      @SuppressWarnings({ "rawtypes" }) Class paramType,
      InboundVariable data,
      Map<String, String> tokens,
      InboundContext inctx,
      String value) throws MarshallException {

    final String typeString = tokens.get("type"); //$NON-NLS-1$
    {
      String[] split = ParseUtil.splitInbound(typeString);
      String splitValue = split[LocalUtil.INBOUND_INDEX_VALUE];
      String splitType = split[LocalUtil.INBOUND_INDEX_TYPE];

      InboundVariable nested = new InboundVariable(data.getLookup(), null, splitType, splitValue);

      final String type = (String) getConverterManager().convertInbound(
          String.class,
          nested,
          inctx,
          null);

      if ("Point".equals(type)) { //$NON-NLS-1$
        return getConverterManager().convertInbound(PointType.class, data, inctx, null);
      }
      else if ("LineString".equals(type)) { //$NON-NLS-1$
        return getConverterManager().convertInbound(LineStringType.class, data, inctx, null);
      }
      else if ("Polygon".equals(type)) { //$NON-NLS-1$
        return getConverterManager().convertInbound(PolygonType.class, data, inctx, null);
      }
      else {
        throw new MarshallException(paramType);
      }
    }
  }

}
