/**
 * Legato is a configurable, lightweight web mapping client that can be
 * easily embedded into web pages and portals, CMS and individual web
 * applications. Legato is implemented in JavaScript and based on the
 * popular open source library OpenLayers.
 *
 * Copyright (C) 2010  disy Informationssysteme GmbH, http://www.disy.net
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package net.disy.ogc.gml.v_3_1_1.dwr;

import javax.xml.bind.JAXBElement;
import javax.xml.namespace.QName;

import net.opengis.gml.v_3_1_1.DirectPositionType;
import net.opengis.gml.v_3_1_1.LineStringType;

import org.directwebremoting.extend.MarshallException;
import org.jvnet.jaxb2_commons.locator.DefaultRootObjectLocator;
import org.jvnet.ogc.gml.v_3_1_1.jts.ConversionFailedException;

import com.vividsolutions.jts.geom.Coordinate;
import com.vividsolutions.jts.geom.LineString;

public class LineStringTypeConverter
    extends
    AbstractGeometryTypeConverter<LineStringType, double[][]> {

  @Override
  protected Class<double[][]> getCoordinatesType() {
    return double[][].class;
  }

  @Override
  protected double[][] createCoordinates(LineStringType geometry) throws MarshallException {

    try {
      final LineString lineString = (LineString) getConverter().createGeometry(
          new DefaultRootObjectLocator(geometry),
          geometry);

      final Coordinate[] lineStringCoordinates = lineString.getCoordinates();

      final double[][] coordinates = new double[lineStringCoordinates.length][];

      for (int index = 0; index < lineStringCoordinates.length; index++) {
        final Coordinate lineStringCoordinate = lineStringCoordinates[index];

        if (Double.isNaN(lineStringCoordinate.z)) {
          coordinates[index] = new double[]{ lineStringCoordinate.x, lineStringCoordinate.y };
        }
        else {
          coordinates[index] = new double[]{
              lineStringCoordinate.x,
              lineStringCoordinate.y,
              lineStringCoordinate.z };
        }
      }
      return coordinates;
    }
    catch (ConversionFailedException cfex) {
      throw new MarshallException(LineStringType.class, cfex);
    }

  }

  @Override
  protected String getGeometryType() {
    return "LineString"; //$NON-NLS-1$
  }

  @Override
  protected LineStringType createGeometry(double[][] coordinates) {
    final LineStringType lineString = new LineStringType();

    for (double[] pos : coordinates) {
      final DirectPositionType position = new DirectPositionType();
      for (double coordinate : pos) {
        position.getValue().add(coordinate);
      }
      lineString.getPosOrPointPropertyOrPointRep().add(

      new JAXBElement<DirectPositionType>(new QName("http://www.opengis.net/gml", "pos"), //$NON-NLS-1$ //$NON-NLS-2$
          DirectPositionType.class,
          position));
    }
    return lineString;
  }
}
