/**
 * Legato is a configurable, lightweight web mapping client that can be
 * easily embedded into web pages and portals, CMS and individual web
 * applications. Legato is implemented in JavaScript and based on the
 * popular open source library OpenLayers.
 *
 * Copyright (C) 2010  disy Informationssysteme GmbH, http://www.disy.net
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package net.disy.ogc.gml.v_3_1_1.dwr;

import javax.xml.bind.JAXBElement;
import javax.xml.namespace.QName;

import net.opengis.gml.v_3_1_1.AbstractRingPropertyType;
import net.opengis.gml.v_3_1_1.DirectPositionType;
import net.opengis.gml.v_3_1_1.LinearRingType;
import net.opengis.gml.v_3_1_1.PolygonType;

import org.directwebremoting.extend.MarshallException;
import org.jvnet.jaxb2_commons.locator.DefaultRootObjectLocator;
import org.jvnet.ogc.gml.v_3_1_1.jts.ConversionFailedException;

import com.vividsolutions.jts.geom.Coordinate;
import com.vividsolutions.jts.geom.LineString;
import com.vividsolutions.jts.geom.Polygon;

public class PolygonTypeConverter extends AbstractGeometryTypeConverter<PolygonType, double[][][]> {

  @Override
  protected double[][][] createCoordinates(PolygonType polygonType) throws MarshallException {

    try {
      final Polygon polygon = (Polygon) getConverter().createGeometry(
          new DefaultRootObjectLocator(polygonType),
          polygonType);

      int interiorRingCount = polygon.getNumInteriorRing();
      int outerSize = 1 + interiorRingCount;

      double[][][] arrayForAllPositions = new double[outerSize][][];

      arrayForAllPositions[0] = extractPositions(polygon.getExteriorRing());

      for (int index = 0; index < interiorRingCount; index++) {
        arrayForAllPositions[1 + index] = extractPositions(polygon.getInteriorRingN(index));
      }

      return arrayForAllPositions;
    }
    catch (ConversionFailedException cfex) {
      throw new MarshallException(PolygonType.class, cfex);
    }
  }

  private double[][] extractPositions(

  LineString ring) {

    final Coordinate[] coordinates = ring.getCoordinates();
    double[][] positions = new double[coordinates.length][];
    for (int index = 0; index < coordinates.length; index++) {
      final Coordinate coordinate = coordinates[index];
      if (Double.isNaN(coordinate.z)) {
        positions[index] = new double[]{ coordinate.x, coordinate.y };
      }
      else {
        positions[index] = new double[]{ coordinate.x, coordinate.y, coordinate.z };
      }

    }
    return positions;
  }

  @Override
  protected String getGeometryType() {
    return "Polygon";
  }

  @Override
  protected PolygonType createGeometry(double[][][] coordinates) {

    final PolygonType polygon = new PolygonType();

    for (int index = 0; index < coordinates.length; index++) {
      final double[][] posList = coordinates[index];

      final LinearRingType linearRing = new LinearRingType();

      for (double[] pos : posList) {
        final DirectPositionType position = new DirectPositionType();
        for (double coordinate : pos) {
          position.getValue().add(coordinate);
        }
        linearRing.getPosOrPointPropertyOrPointRep().add(

        new JAXBElement<DirectPositionType>(new QName("http://www.opengis.net/gml", "pos"), //$NON-NLS-1$ //$NON-NLS-2$
            DirectPositionType.class,
            position));
      }

      if (index == 0) {
        AbstractRingPropertyType ringPropertyType = new AbstractRingPropertyType();

        ringPropertyType.setRing(new JAXBElement<LinearRingType>(new QName(
            "http://www.opengis.net/gml", //$NON-NLS-1$
            "LinearRing"), LinearRingType.class, linearRing)); //$NON-NLS-1$

        polygon.setExterior(new JAXBElement<AbstractRingPropertyType>(new QName(
            "http://www.opengis.net/gml", //$NON-NLS-1$
            "exterior"), AbstractRingPropertyType.class, ringPropertyType)); //$NON-NLS-1$
      }
      else {
        AbstractRingPropertyType ringPropertyType = new AbstractRingPropertyType();

        ringPropertyType.setRing(new JAXBElement<LinearRingType>(new QName(
            "http://www.opengis.net/gml", //$NON-NLS-1$
            "LinearRing"), LinearRingType.class, linearRing)); //$NON-NLS-1$

        polygon.getInterior().add(
            new JAXBElement<AbstractRingPropertyType>(new QName("http://www.opengis.net/gml", //$NON-NLS-1$
                "interior"), AbstractRingPropertyType.class, ringPropertyType)); //$NON-NLS-1$

      }
    }
    return polygon;

  }

  @Override
  protected Class<double[][][]> getCoordinatesType() {
    return double[][][].class;
  }
}
