/**
 * Legato is a configurable, lightweight web mapping client that can be
 * easily embedded into web pages and portals, CMS and individual web
 * applications. Legato is implemented in JavaScript and based on the
 * popular open source library OpenLayers.
 *
 * Copyright (C) 2010  disy Informationssysteme GmbH, http://www.disy.net
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package net.disy.ogc.gml.v_3_1_1.jts.converter;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBElement;
import javax.xml.bind.JAXBException;
import javax.xml.transform.dom.DOMResult;

import net.disy.ogc.gml.v_3_1_1.GmlConstants;
import net.disy.ogc.wps.v_1_0_0.converter.AbstractConverter;
import net.disy.ogc.wps.v_1_0_0.converter.TypeConverter;
import net.opengis.gml.v_3_1_1.AbstractGeometryType;

import org.apache.commons.lang.Validate;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import com.vividsolutions.jts.geom.Geometry;

public class GeometryDOMElementTypeConverter extends AbstractConverter<Element, Geometry> {

  private final JAXBContext context;
  private final TypeConverter<JAXBElement<? extends AbstractGeometryType>, Geometry> typeConverter;

  public GeometryDOMElementTypeConverter() {
    final JAXBContext context;
    try {
      context = JAXBContext.newInstance(GmlConstants.CONTEXT_PATH);
    }
    catch (JAXBException jaxbex) {
      throw new ExceptionInInitializerError(jaxbex);
    }
    this.context = context;
    this.typeConverter = new GeometryJAXBElementTypeConverter();
  }

  protected GeometryDOMElementTypeConverter(
      JAXBContext context,
      TypeConverter<JAXBElement<? extends AbstractGeometryType>, Geometry> typeConverter) {
    Validate.notNull(context);
    Validate.notNull(typeConverter);
    this.context = context;
    this.typeConverter = typeConverter;
  }

  @Override
  protected Element convertFromNotNull(Geometry source) {
    try {
      final JAXBElement<? extends AbstractGeometryType> value = typeConverter.convertFrom(source);
      final DOMResult result = new DOMResult();
      context.createMarshaller().marshal(value, result);
      return ((Document) result.getNode()).getDocumentElement();
    }
    catch (JAXBException jaxbex) {
      throw new IllegalArgumentException(jaxbex);
    }
  }

  @Override
  protected Geometry convertToNotNull(Element source) {
    try {
      @SuppressWarnings("unchecked")
      final JAXBElement<? extends AbstractGeometryType> value = (JAXBElement<? extends AbstractGeometryType>) context
          .createUnmarshaller()
          .unmarshal(source);
      return typeConverter.convertTo(value);
    }
    catch (JAXBException jaxbex) {
      throw new IllegalArgumentException(jaxbex);
    }
  }

  @Override
  public Class<Geometry> getDestinationClass() {
    return Geometry.class;
  }

  @Override
  public Class<Element> getSourceClass() {
    return Element.class;
  }
}
