/**
 * Legato is a configurable, lightweight web mapping client that can be
 * easily embedded into web pages and portals, CMS and individual web
 * applications. Legato is implemented in JavaScript and based on the
 * popular open source library OpenLayers.
 *
 * Copyright (C) 2010  disy Informationssysteme GmbH, http://www.disy.net
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package net.disy.ogc.gml.v_3_1_1.jts.converter;

import javax.xml.bind.JAXBElement;

import net.disy.ogc.wps.v_1_0_0.converter.AbstractConverter;
import net.disy.ogc.wps.v_1_0_0.converter.TypeConverter;
import net.opengis.gml.v_3_1_1.AbstractGeometricAggregateType;
import net.opengis.gml.v_3_1_1.AbstractGeometryType;
import net.opengis.gml.v_3_1_1.LineStringType;
import net.opengis.gml.v_3_1_1.LinearRingType;
import net.opengis.gml.v_3_1_1.MultiGeometryType;
import net.opengis.gml.v_3_1_1.MultiLineStringType;
import net.opengis.gml.v_3_1_1.MultiPointType;
import net.opengis.gml.v_3_1_1.MultiPolygonType;
import net.opengis.gml.v_3_1_1.ObjectFactory;
import net.opengis.gml.v_3_1_1.PointType;
import net.opengis.gml.v_3_1_1.PolygonType;

import org.apache.commons.lang.Validate;
import org.jvnet.ogc.gml.v_3_1_1.ObjectFactoryInterface;
import org.jvnet.ogc.gml.v_3_1_1.jts.GML311ToJTSSRIDConverterInterface;
import org.jvnet.ogc.gml.v_3_1_1.jts.JTSToGML311Constants;
import org.jvnet.ogc.gml.v_3_1_1.jts.JTSToGML311SRSReferenceGroupConverterInterface;
import org.jvnet.ogc.gml.v_3_1_1.jts.NullGML311ToJTSSRIDConverter;

import com.vividsolutions.jts.geom.Geometry;
import com.vividsolutions.jts.geom.GeometryFactory;

public class GeometryJAXBElementTypeConverter
    extends
    AbstractConverter<JAXBElement<? extends AbstractGeometryType>, Geometry> {

  private final TypeConverter<AbstractGeometryType, Geometry> typeConverter;
  private final ObjectFactoryInterface objectFactory;

  public GeometryJAXBElementTypeConverter() {
    this(
        JTSToGML311Constants.DEFAULT_OBJECT_FACTORY,
        JTSToGML311Constants.DEFAULT_SRS_REFERENCE_GROUP_CONVERTER,
        new GeometryFactory(),
        new NullGML311ToJTSSRIDConverter());
  }

  public GeometryJAXBElementTypeConverter(
      ObjectFactoryInterface objectFactory,
      JTSToGML311SRSReferenceGroupConverterInterface srsReferenceGroupConverter,
      GeometryFactory geometryFactory,
      GML311ToJTSSRIDConverterInterface sridConverter) {
    this(objectFactory, new GeometryTypeConverter(
        objectFactory,
        srsReferenceGroupConverter,
        geometryFactory,
        sridConverter));
  }

  protected GeometryJAXBElementTypeConverter(
      ObjectFactoryInterface objectFactory,
      TypeConverter<AbstractGeometryType, Geometry> typeConverter) {
    Validate.notNull(objectFactory);
    Validate.notNull(typeConverter);
    this.objectFactory = objectFactory;
    this.typeConverter = typeConverter;
  }

  @Override
  public Class<JAXBElement<? extends AbstractGeometryType>> getSourceClass() {
    final Class<?> jaxbElementClass = JAXBElement.class;
    @SuppressWarnings("unchecked")
    final Class<JAXBElement<? extends AbstractGeometryType>> sourceClass = (Class<JAXBElement<? extends AbstractGeometryType>>) jaxbElementClass;
    return sourceClass;
  }

  @Override
  public Class<Geometry> getDestinationClass() {
    return Geometry.class;
  }

  @Override
  protected JAXBElement<? extends AbstractGeometryType> convertFromNotNull(Geometry source) {
    final AbstractGeometryType abstractGeometryType = typeConverter.convertFrom(source);

    if (abstractGeometryType instanceof PointType) {
      return objectFactory.createPoint((PointType) abstractGeometryType);
    }
    else if (abstractGeometryType instanceof PolygonType) {
      return objectFactory.createPolygon((PolygonType) abstractGeometryType);
    }
    else if (abstractGeometryType instanceof LineStringType) {
      return objectFactory.createLineString((LineStringType) abstractGeometryType);
    }
    else if (abstractGeometryType instanceof LinearRingType) {
      return objectFactory.createLinearRing((LinearRingType) abstractGeometryType);
    }
    else if (abstractGeometryType instanceof MultiPointType) {
      return objectFactory.createMultiPoint((MultiPointType) abstractGeometryType);
    }
    else if (abstractGeometryType instanceof MultiLineStringType) {
      return objectFactory.createMultiLineString((MultiLineStringType) abstractGeometryType);
    }
    else if (abstractGeometryType instanceof MultiPolygonType) {
      return objectFactory.createMultiPolygon((MultiPolygonType) abstractGeometryType);
    }
    else if (abstractGeometryType instanceof MultiGeometryType) {
      return objectFactory.createMultiGeometry((MultiGeometryType) abstractGeometryType);
    }
    else {
      throw new IllegalArgumentException("Unexpected geometric type [" + source.getClass() + "]."); //$NON-NLS-1$ //$NON-NLS-2$
    }
  }

  @Override
  protected Geometry convertToNotNull(JAXBElement<? extends AbstractGeometryType> source) {
    final AbstractGeometryType value = source.getValue();
    Validate.notNull(value);
    return typeConverter.convertTo(value);
  }

}
