/**
 * Legato is a configurable, lightweight web mapping client that can be
 * easily embedded into web pages and portals, CMS and individual web
 * applications. Legato is implemented in JavaScript and based on the
 * popular open source library OpenLayers.
 *
 * Copyright (C) 2010  disy Informationssysteme GmbH, http://www.disy.net
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package net.disy.ogc.gml.v_3_1_1.jts.converter;

import net.disy.ogc.wps.v_1_0_0.converter.TypeConverter;
import net.opengis.gml.v_3_1_1.AbstractGeometryType;

import org.apache.commons.lang.Validate;
import org.jvnet.jaxb2_commons.locator.DefaultRootObjectLocator;
import org.jvnet.ogc.gml.v_3_1_1.ObjectFactoryInterface;
import org.jvnet.ogc.gml.v_3_1_1.jts.ConversionFailedException;
import org.jvnet.ogc.gml.v_3_1_1.jts.GML311ToJTSGeometryConverter;
import org.jvnet.ogc.gml.v_3_1_1.jts.GML311ToJTSSRIDConverterInterface;
import org.jvnet.ogc.gml.v_3_1_1.jts.JTSToGML311Constants;
import org.jvnet.ogc.gml.v_3_1_1.jts.JTSToGML311GeometryConverter;
import org.jvnet.ogc.gml.v_3_1_1.jts.JTSToGML311SRSReferenceGroupConverterInterface;
import org.jvnet.ogc.gml.v_3_1_1.jts.NullGML311ToJTSSRIDConverter;

import com.vividsolutions.jts.geom.Geometry;
import com.vividsolutions.jts.geom.GeometryFactory;

public class GeometryTypeConverter implements TypeConverter<AbstractGeometryType, Geometry> {

  private final JTSToGML311GeometryConverter toConverter;

  private final GML311ToJTSGeometryConverter fromConverter;

  public GeometryTypeConverter() {
    this(
        JTSToGML311Constants.DEFAULT_OBJECT_FACTORY,
        JTSToGML311Constants.DEFAULT_SRS_REFERENCE_GROUP_CONVERTER,
        new GeometryFactory(),
        new NullGML311ToJTSSRIDConverter());
  }

  public GeometryTypeConverter(
      ObjectFactoryInterface objectFactory,
      JTSToGML311SRSReferenceGroupConverterInterface srsReferenceGroupConverter,
      GeometryFactory geometryFactory,
      GML311ToJTSSRIDConverterInterface sridConverter) {
    Validate.notNull(objectFactory);
    Validate.notNull(geometryFactory);
    this.toConverter = new JTSToGML311GeometryConverter(objectFactory, srsReferenceGroupConverter);
    this.fromConverter = new GML311ToJTSGeometryConverter(geometryFactory, sridConverter);
  }

  @Override
  public AbstractGeometryType convertFrom(Geometry destination) {
    return this.toConverter.createGeometryType(destination);
  }

  @Override
  public Geometry convertTo(AbstractGeometryType source) {
    try {
      return this.fromConverter.createGeometry(new DefaultRootObjectLocator(source), source);
    }
    catch (ConversionFailedException cfex) {
      throw new IllegalArgumentException(cfex);
    }
  }

  @Override
  public Class<Geometry> getDestinationClass() {
    return Geometry.class;
  }

  @Override
  public Class<AbstractGeometryType> getSourceClass() {
    return AbstractGeometryType.class;
  }

}
