/**
 * Legato is a configurable, lightweight web mapping client that can be
 * easily embedded into web pages and portals, CMS and individual web
 * applications. Legato is implemented in JavaScript and based on the
 * popular open source library OpenLayers.
 *
 * Copyright (C) 2010  disy Informationssysteme GmbH, http://www.disy.net
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package net.disy.ogc.ows.v_1_1_0.util;

import javax.xml.bind.JAXBElement;
import javax.xml.namespace.QName;

import net.opengis.ows.v_1_1_0.CodeType;
import net.opengis.ows.v_1_1_0.DCP;
import net.opengis.ows.v_1_1_0.HTTP;
import net.opengis.ows.v_1_1_0.Operation;
import net.opengis.ows.v_1_1_0.RequestMethodType;

import org.apache.commons.lang.Validate;

public class OwsUtils {

  private OwsUtils() {
    //do nothing
  }

  public static String toString(CodeType code) {
    if (code == null) {
      return null;
    }
    return (code.getCodeSpace() == null ? "" : '{' + code.getCodeSpace() + '}') + code.getValue(); //$NON-NLS-1$
  }

  @SuppressWarnings("unchecked")
  public static JAXBElement<RequestMethodType> createGet(String uri) {
    Validate.notNull(uri);
    final RequestMethodType requestMethodType = createRequestMethodType(uri);
    return new JAXBElement(new QName("http://www.opengis.net/ows/1.1", "Get"), //$NON-NLS-1$ //$NON-NLS-2$
        RequestMethodType.class,
        HTTP.class,
        requestMethodType);
  }

  @SuppressWarnings("unchecked")
  public static JAXBElement<RequestMethodType> createPost(String uri) {
    Validate.notNull(uri);
    final RequestMethodType requestMethodType = createRequestMethodType(uri);
    return new JAXBElement(new QName("http://www.opengis.net/ows/1.1", "Post"), //$NON-NLS-1$ //$NON-NLS-2$
        RequestMethodType.class,
        HTTP.class,
        requestMethodType);
  }

  public static RequestMethodType createRequestMethodType(String uri) {
    final RequestMethodType requestMethodType = new RequestMethodType();
    requestMethodType.setHref(uri);
    requestMethodType.setType("simple"); //$NON-NLS-1$
    return requestMethodType;
  }

  public static Operation createOperation(String name) {
    Validate.notNull(name);

    final Operation operation = new Operation();
    operation.setName(name);
    final DCP dcp = new DCP();
    operation.getDCP().add(dcp);
    final HTTP http = new HTTP();
    dcp.setHTTP(http);
    return operation;
  }
}
