/**
 * Legato is a configurable, lightweight web mapping client that can be
 * easily embedded into web pages and portals, CMS and individual web
 * applications. Legato is implemented in JavaScript and based on the
 * popular open source library OpenLayers.
 *
 * Copyright (C) 2010  disy Informationssysteme GmbH, http://www.disy.net
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package net.disy.ogc.wps.v_1_0_0;

import java.util.Collections;
import java.util.Locale;

import net.disy.commons.servlet.util.UrlProvider;
import net.opengis.ows.v_1_1_0.ServiceIdentification;
import net.opengis.ows.v_1_1_0.ServiceProvider;

public class DefaultWpsOperationsFactory {

  private final ServiceIdentification serviceIdentification;

  private final ServiceProvider serviceProvider;

  private Locale defaultLanguage = Locale.ENGLISH;

  private final WpsProcessRegistry wpsProcessRegistry;

  private final String servletName;

  public DefaultWpsOperationsFactory(
      ServiceIdentification serviceIdentification,
      ServiceProvider serviceProvider,
      Locale defaultLocale,
      WpsProcessRegistry processRegistry,
      final String servletName) {
    this.serviceIdentification = serviceIdentification;
    this.serviceProvider = serviceProvider;
    this.defaultLanguage = defaultLocale;
    this.wpsProcessRegistry = processRegistry;
    this.servletName = servletName;
  }

  public ServiceIdentification getServiceIdentification() {
    return serviceIdentification;
  }

  public ServiceProvider getServiceProvider() {
    return serviceProvider;
  }

  public Locale getDefaultLanguage() {
    return defaultLanguage;
  }

  public WpsProcessRegistry getWpsProcessRegistry() {
    return wpsProcessRegistry;
  }

  public WpsOperations createWpsOperations(UrlProvider urlProvider) {

    final DefaultGetCapabilitiesService getCapabilitiesService = new DefaultGetCapabilitiesService();

    getCapabilitiesService.setWpsProcessRegistry(getWpsProcessRegistry());
    getCapabilitiesService.setDefaultLanguage(getDefaultLanguage());
    getCapabilitiesService.setSupportedLanguages(Collections.singletonList(getDefaultLanguage()));
    getCapabilitiesService.setOperationsMetadata(new DefaultOperationsMetadataFactory(urlProvider
        .getUrl(servletName)).createOperationsMetadata());
    getCapabilitiesService.setServiceIdentification(getServiceIdentification());
    getCapabilitiesService.setServiceProvider(getServiceProvider());

    final DefaultDescribeProcessService describeProcessService = new DefaultDescribeProcessService();
    describeProcessService.setWpsProcessRegistry(getWpsProcessRegistry());
    describeProcessService.setDefaultLanguage(getDefaultLanguage());

    final DefaultExecuteService executeService = new DefaultExecuteService(
        urlProvider.getUrl(servletName));
    executeService.setWpsProcessRegistry(getWpsProcessRegistry());

    final DefaultWpsOperations wpsOperations = new DefaultWpsOperations();

    wpsOperations.setGetCapabilitiesService(getCapabilitiesService);
    wpsOperations.setDescribeProcessService(describeProcessService);
    wpsOperations.setExecuteService(executeService);
    return wpsOperations;

  }

}
