/**
 * Legato is a configurable, lightweight web mapping client that can be
 * easily embedded into web pages and portals, CMS and individual web
 * applications. Legato is implemented in JavaScript and based on the
 * popular open source library OpenLayers.
 *
 * Copyright (C) 2010  disy Informationssysteme GmbH, http://www.disy.net
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package net.disy.ogc.wps.v_1_0_0.annotation;

import java.util.List;

import net.disy.ogc.wps.v_1_0_0.procedure.AbstractAnnotationParameterDescription;
import net.disy.ogc.wps.v_1_0_0.procedure.InputParameterDescription;
import net.disy.ogc.wps.v_1_0_0.procedure.ParameterType;
import net.disy.ogc.wps.v_1_0_0.util.WpsProcessUtilities;
import net.opengis.ows.v_1_1_0.MetadataType;

public class AnnotationInputParameterDescription extends AbstractAnnotationParameterDescription
    implements
    InputParameterDescription {

  private final InputParameter inputParameter;

  public AnnotationInputParameterDescription(Class<?> parameterType, InputParameter inputParameter) {
    this(parameterType, inputParameter, null);
  }

  public AnnotationInputParameterDescription(
      Class<?> parameterType,
      InputParameter inputParameter,
      Object defaultValue) {
    super(parameterType);
    this.inputParameter = inputParameter;
    this.defaultValue = defaultValue;
  }

  @Override
  public String getId() {
    return inputParameter.id();
  }

  @Override
  public String getTitle() {
    return inputParameter.title();
  }

  @Override
  public String getAbstract() {
    return inputParameter.description();
  }

  @Deprecated
  @Override
  public ParameterType getParameterType() {
    return inputParameter.type();
  }
  
  @Override
  public DataType getDataType() {
    return inputParameter.dataType();
  }

  @Override
  public boolean isOptional() {
    return inputParameter.optional();
  }

  @Override
  public List<MetadataType> getMetadata() {
    return WpsProcessUtilities.getMetadata(inputParameter.metadata());
  }

  private Object defaultValue;

  @Override
  public Object getDefaultValue() {
    return defaultValue;
  }
}
