/**
 * Legato is a configurable, lightweight web mapping client that can be
 * easily embedded into web pages and portals, CMS and individual web
 * applications. Legato is implemented in JavaScript and based on the
 * popular open source library OpenLayers.
 *
 * Copyright (C) 2010  disy Informationssysteme GmbH, http://www.disy.net
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package net.disy.ogc.wps.v_1_0_0.annotation;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.text.MessageFormat;
import java.util.List;

import org.apache.commons.lang.Validate;

import net.disy.ogc.ows.v_1_1_0.OwsException;
import net.disy.ogc.wps.v_1_0_0.WpsProcess;
import net.disy.ogc.wps.v_1_0_0.WpsProcessRegistry;
import net.disy.ogc.wps.v_1_0_0.procedure.WpsProcessContext;
import net.disy.ogc.wps.v_1_0_0.sample.AnnotationProcedureWpsProcess;

public class ProcessForClassNameAdder {

  private final WpsProcessContext wpsProcessContext;

  private final WpsProcessRegistry wpsProcessRegistry;

  public ProcessForClassNameAdder(
      WpsProcessContext wpsProcessContext,
      WpsProcessRegistry wpsProcessRegistry) {
    Validate.notNull(wpsProcessContext);
    Validate.notNull(wpsProcessRegistry);
    this.wpsProcessContext = wpsProcessContext;
    this.wpsProcessRegistry = wpsProcessRegistry;
  }

  public void addProcessesForClassNames(final List<String> processClassNames) {
    for (String processClassName : processClassNames) {
      addProcessForClassName(processClassName);
    }
  }

  private void addProcessForClassName(String processClassName) {
    try {
      if (processClassName != null) {
        final Class<?> processClass = Class.forName(processClassName.trim());

        Object draftProcess = null;

        //        try
        //        {
        try {
          Constructor<?> defaultConstructor = processClass.getConstructor(WpsProcessContext.class);
          draftProcess = defaultConstructor.newInstance(this.wpsProcessContext);
        }
        catch (NoSuchMethodException nsmex1) {
          try {
            Constructor<?> contextConstructor = processClass.getConstructor();
            draftProcess = contextConstructor.newInstance();
          }
          catch (NoSuchMethodException nsmex2) {
            throw new IllegalArgumentException(
                MessageFormat
                    .format(
                        "Process class [{0}] must declare a default constructor or a constructor accepting and instance of [{1}].",
                        processClass.getName(),
                        WpsProcessContext.class.getName()));
          }
        }
        //        }
        //        catch(InstantiationException iex)
        //        {
        //          throw new IllegalArgumentException("Could not instantiate the process class.", iex);
        //        }
        //        catch(InvocationTargetException itex)
        //        {
        //          throw new IllegalArgumentException("Could not instantiate the process class.", itex);
        //        }

        WpsProcess wpsProcess;
        if (draftProcess instanceof WpsProcess) {
          wpsProcess = (WpsProcess) draftProcess;
        }
        else {
          wpsProcess = new AnnotationProcedureWpsProcess(this.wpsProcessContext, draftProcess);
        }
        wpsProcessRegistry.registerProcess(wpsProcess);
      }
    }
    catch (ClassNotFoundException cnfex) {
      throw new IllegalArgumentException(
          "Could not find process class [" + processClassName + "].", //$NON-NLS-1$ //$NON-NLS-2$
          cnfex);
    }
    catch (InstantiationException iex) {
      throw new IllegalArgumentException("Could not instantiate process class [" //$NON-NLS-1$
          + processClassName
          + "].", iex); //$NON-NLS-1$
    }
    catch (InvocationTargetException itex) {
      throw new IllegalArgumentException("Could not instantiate process class [" //$NON-NLS-1$
          + processClassName
          + "].", itex); //$NON-NLS-1$
    }
    catch (IllegalAccessException iaex) {
      throw new IllegalArgumentException("Could not instantiate process class [" //$NON-NLS-1$
          + processClassName
          + "].", iaex); //$NON-NLS-1$
    }
    catch (OwsException owsException) {
      throw new IllegalArgumentException(
          "Could not register the process [" + processClassName + "].", //$NON-NLS-1$ //$NON-NLS-2$
          owsException);
    }
  }
}
