/**
 * Legato is a configurable, lightweight web mapping client that can be
 * easily embedded into web pages and portals, CMS and individual web
 * applications. Legato is implemented in JavaScript and based on the
 * popular open source library OpenLayers.
 *
 * Copyright (C) 2010  disy Informationssysteme GmbH, http://www.disy.net
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package net.disy.ogc.wps.v_1_0_0.converter;

import static net.disy.ogc.wps.v_1_0_0.util.Validate.*;
import static org.apache.commons.lang.Validate.*;

import java.util.HashMap;
import java.util.Map;

public class DefaultTypeConverterRegistry implements TypeConverterRegistry {
  private final Map<Class<?>, DefaultTypeConverterForSource<?>> typeConverters = new HashMap<Class<?>, DefaultTypeConverterForSource<?>>();

  public <S> void addConverter(TypeConverter<S, ?> typeConverter) {
    notNull(typeConverter);
    getTypeConverter(typeConverter.getSourceClass()).addConverter(typeConverter);
  }

  @Override
  public <S, D> D convertTo(Class<S> sourceClass, Class<D> destinationClass, S object) {
    notNull(sourceClass, destinationClass);
    return getTypeConverter(sourceClass).convertTo(destinationClass, object);
  }

  @Override
  public <S, D> S convertFrom(Class<S> sourceClass, Class<D> destinationClass, D object) {
    notNull(sourceClass, destinationClass);
    return getTypeConverter(sourceClass).convertFrom(destinationClass, object);
  }

  @SuppressWarnings("unchecked")
  private <S> DefaultTypeConverterForSource<S> getTypeConverter(Class<S> sourceClass) {
    if (!typeConverters.containsKey(sourceClass)) {
      typeConverters.put(sourceClass, new DefaultTypeConverterForSource<S>(sourceClass));
    }

    return (DefaultTypeConverterForSource<S>) typeConverters.get(sourceClass);
  }

  @Override
  public boolean isConversionSupported(Class<?> sourceClass, Class<?> destinationClass) {
    notNull(sourceClass, destinationClass);
    return getTypeConverter(sourceClass).isConversionSupported(destinationClass);
  }

}
