/**
 * Legato is a configurable, lightweight web mapping client that can be
 * easily embedded into web pages and portals, CMS and individual web
 * applications. Legato is implemented in JavaScript and based on the
 * popular open source library OpenLayers.
 *
 * Copyright (C) 2010  disy Informationssysteme GmbH, http://www.disy.net
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package net.disy.ogc.wps.v_1_0_0.converter;

import javax.xml.bind.JAXBElement;
import javax.xml.namespace.QName;

import org.apache.commons.lang.Validate;

public class JAXBElementTypeConverter<T> extends AbstractConverter<JAXBElement<? extends T>, T> {

  private final Class<T> destinationClass;
  private final QName name;

  public JAXBElementTypeConverter(Class<T> destinationClass, QName name) {
    super();
    Validate.notNull(destinationClass);
    Validate.notNull(name);
    this.destinationClass = destinationClass;
    this.name = name;
  }

  public QName getName() {
    return name;
  }

  @Override
  @SuppressWarnings("unchecked")
  public Class<JAXBElement<? extends T>> getSourceClass() {
    final Class c = JAXBElement.class;
    return c;
  }

  @Override
  public Class<T> getDestinationClass() {
    return destinationClass;
  }

  @Override
  protected T convertToNotNull(JAXBElement source) {
    return (T) source.getValue();
  }

  @Override
  protected JAXBElement convertFromNotNull(T source) {
    return new JAXBElement<T>(getName(), getDestinationClass(), source);
  }

}
