/**
 * Legato is a configurable, lightweight web mapping client that can be
 * easily embedded into web pages and portals, CMS and individual web
 * applications. Legato is implemented in JavaScript and based on the
 * popular open source library OpenLayers.
 *
 * Copyright (C) 2010  disy Informationssysteme GmbH, http://www.disy.net
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package net.disy.ogc.wps.v_1_0_0.converter;

import org.apache.commons.lang.ClassUtils;
import org.apache.commons.lang.Validate;

public class PrimitiveTypeConverter<T> implements TypeConverter<T, T> {

  private final Class<T> sourceClass;
  private final Class<T> destinationClass;

  public PrimitiveTypeConverter(Class<T> primitiveClass) {
    Validate.notNull(primitiveClass);
    Validate.isTrue(primitiveClass.isPrimitive(), "The class ["
        + primitiveClass
        + "] is not a primitive class.");
    @SuppressWarnings("unchecked")
    final Class<T> wrapperClass = ClassUtils.primitiveToWrapper(primitiveClass);
    this.sourceClass = wrapperClass;
    this.destinationClass = primitiveClass;
  }

  @Override
  public T convertFrom(T value) {
    Validate.notNull(value);
    return value;
  }

  @Override
  public T convertTo(T value) {
    Validate.notNull(value);
    return value;
  }

  @Override
  public Class<T> getDestinationClass() {
    return destinationClass;
  }

  @Override
  public Class<T> getSourceClass() {
    return sourceClass;
  }

}
