/**
 * Legato is a configurable, lightweight web mapping client that can be
 * easily embedded into web pages and portals, CMS and individual web
 * applications. Legato is implemented in JavaScript and based on the
 * popular open source library OpenLayers.
 *
 * Copyright (C) 2010  disy Informationssysteme GmbH, http://www.disy.net
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package net.disy.ogc.wps.v_1_0_0.converter;

public class ReducedConverter<S, D> implements TypeConverter<S, D> {
  private final Class<D> destinationClass;
  private final TypeConverter<S, ? super D> converter;

  public ReducedConverter(Class<D> destinationClass, TypeConverter<S, ? super D> converter) {
    this.destinationClass = destinationClass;
    this.converter = converter;
  }

  @Override
  public Class<S> getSourceClass() {
    return converter.getSourceClass();
  }

  @Override
  public Class<D> getDestinationClass() {
    return destinationClass;
  }

  @Override
  public D convertTo(S source) {
    Object result = converter.convertTo(source);
    if (result == null) {
      return null;
    }

    if (!destinationClass.isAssignableFrom(result.getClass())) {
      throw new IllegalArgumentException("Object '"
          + result
          + "' is no instance of '"
          + destinationClass
          + "'.");
    }

    return (D) result;
  }

  @Override
  public S convertFrom(D destination) {
    S result = converter.convertFrom(destination);
    if (result == null) {
      return null;
    }

    return result;
  }
}
