/**
 * Legato is a configurable, lightweight web mapping client that can be
 * easily embedded into web pages and portals, CMS and individual web
 * applications. Legato is implemented in JavaScript and based on the
 * popular open source library OpenLayers.
 *
 * Copyright (C) 2010  disy Informationssysteme GmbH, http://www.disy.net
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package net.disy.ogc.wps.v_1_0_0.converter;

import java.text.MessageFormat;
import java.util.Collection;
import java.util.Collections;
import java.util.List;

import org.apache.commons.collections15.CollectionUtils;
import org.apache.commons.collections15.Predicate;
import org.apache.commons.lang.StringUtils;

public class SingleItemTypeConverter<D> extends AbstractConverter<List<Object>, D> {

  private final Class<D> destinationClass;

  public SingleItemTypeConverter(Class<D> destinationClass) {
    this.destinationClass = destinationClass;
  }

  @Override
  protected List<Object> convertFromNotNull(D source) {
    return Collections.<Object> singletonList(source);
  }

  @Override
  protected D convertToNotNull(List<Object> source) {

    final Collection<Object> sourceWithoutWhitespaceStrings = CollectionUtils.select(
        source,
        new Predicate<Object>() {

          @Override
          public boolean evaluate(Object object) {
            if (object == null) {
              return false;
            }
            else if (object instanceof String) {
              return !StringUtils.isWhitespace((String) object);
            }
            else {
              return true;
            }
          }
        });
    if (sourceWithoutWhitespaceStrings.size() != 1) {
      throw new IllegalArgumentException("Source list must have exactly one JAXB element."); //$NON-NLS-1$
    }
    final Object item = sourceWithoutWhitespaceStrings.iterator().next();
    final Class<? extends Object> itemClass = item.getClass();
    if (this.destinationClass.isAssignableFrom(itemClass)) {
      @SuppressWarnings("unchecked")
      final D result = (D) item;
      return result;
    }
    else {
      throw new IllegalArgumentException(MessageFormat.format(
          "Source list has an element of the wrong type [{0}].", itemClass.getName())); //$NON-NLS-1$
    }
  }

  @Override
  public Class<D> getDestinationClass() {
    return destinationClass;
  }

  @Override
  public Class<List<Object>> getSourceClass() {
    final Class<?> theClass = List.class;
    @SuppressWarnings("unchecked")
    final Class<List<Object>> sourceClass = (Class<List<Object>>) theClass;
    return sourceClass;
  }

}
