/**
 * Legato is a configurable, lightweight web mapping client that can be
 * easily embedded into web pages and portals, CMS and individual web
 * applications. Legato is implemented in JavaScript and based on the
 * popular open source library OpenLayers.
 *
 * Copyright (C) 2010  disy Informationssysteme GmbH, http://www.disy.net
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package net.disy.ogc.wps.v_1_0_0.converter;

import java.util.Collection;
import java.util.Collections;
import java.util.List;

import javax.xml.bind.JAXBElement;

import org.apache.commons.collections15.CollectionUtils;
import org.apache.commons.collections15.Predicate;
import org.apache.commons.lang.StringUtils;
import org.apache.commons.lang.Validate;

public class SingleJAXBElementTypeConverter<V, D> extends AbstractConverter<List<Object>, D> {

  private TypeConverter<JAXBElement<? extends V>, D> jaxbElementTypeConverter;

  public SingleJAXBElementTypeConverter(
      TypeConverter<JAXBElement<? extends V>, D> jaxbElementTypeConverter) {
    super();
    Validate.notNull(jaxbElementTypeConverter);
    this.jaxbElementTypeConverter = jaxbElementTypeConverter;
  }

  @Override
  protected List<Object> convertFromNotNull(D source) {
    return Collections.<Object> singletonList(jaxbElementTypeConverter.convertFrom(source));
  }

  @Override
  protected D convertToNotNull(List<Object> source) {

    final Collection<Object> sourceWithoutWhitespaceStrings = CollectionUtils.select(
        source,
        new Predicate<Object>() {

          @Override
          public boolean evaluate(Object object) {
            if (object == null) {
              return false;
            }
            else if (object instanceof String) {
              return !StringUtils.isWhitespace((String) object);
            }
            else {
              return true;
            }
          }
        });
    if (sourceWithoutWhitespaceStrings.size() != 1) {
      throw new IllegalArgumentException("Source list must have exactly one JAXB element."); //$NON-NLS-1$
    }
    final Object item = sourceWithoutWhitespaceStrings.iterator().next();
    if (item instanceof JAXBElement) {
      @SuppressWarnings("unchecked")
      final JAXBElement<? extends V> element = (JAXBElement<? extends V>) item;
      return jaxbElementTypeConverter.convertTo(element);
    }
    else {
      throw new IllegalArgumentException("Source list must have exactly one JAXB element."); //$NON-NLS-1$
    }
  }

  @Override
  public Class<D> getDestinationClass() {
    return jaxbElementTypeConverter.getDestinationClass();
  }

  @Override
  public Class<List<Object>> getSourceClass() {
    final Class<?> theClass = List.class;
    @SuppressWarnings("unchecked")
    final Class<List<Object>> sourceClass = (Class<List<Object>>) theClass;
    return sourceClass;
  }

}
