/**
 * Legato is a configurable, lightweight web mapping client that can be
 * easily embedded into web pages and portals, CMS and individual web
 * applications. Legato is implemented in JavaScript and based on the
 * popular open source library OpenLayers.
 *
 * Copyright (C) 2010  disy Informationssysteme GmbH, http://www.disy.net
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package net.disy.ogc.wps.v_1_0_0.dwr;

import java.text.MessageFormat;

import javax.servlet.http.HttpSession;

import org.directwebremoting.annotations.RemoteProxy;

import net.disy.ogc.wps.v_1_0_0.WpsInception;
import net.disy.ogc.wps.v_1_0_0.bean.WpsInceptionBean;
import net.disy.ogc.wps.v_1_0_0.util.Validate;

@RemoteProxy(name = "WpsInception")
public class WpsInceptionController extends AbstractWpsInceptionController {

  private final String beanName;

  public WpsInceptionController(String beanName) {
    Validate.notNull(beanName);
    this.beanName = beanName;
  }

  public WpsInceptionController() {
    this(WpsInceptionBean.DEFAULT_BEAN_NAME);
  }

  @Override
  protected WpsInception getWpsInception(HttpSession session) {
    Object draftBean = session.getAttribute(beanName);
    if (draftBean == null) {
      return null;
    }
    else if (draftBean instanceof WpsInception) {
      return (WpsInception) draftBean;
    }
    else {
      throw new IllegalStateException(MessageFormat.format(
          "WPS inception bean [{0}] does not implement the [{1}] interface.",
          beanName,
          WpsInception.class.getName()));
    }
  }

}
