/**
 * Legato is a configurable, lightweight web mapping client that can be
 * easily embedded into web pages and portals, CMS and individual web
 * applications. Legato is implemented in JavaScript and based on the
 * popular open source library OpenLayers.
 *
 * Copyright (C) 2010  disy Informationssysteme GmbH, http://www.disy.net
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package net.disy.ogc.wps.v_1_0_0.model;

import static org.apache.commons.lang.Validate.notNull;

import java.util.LinkedList;
import java.util.List;

public class DefaultCRSRegistry implements CRSRegistry {

  private CRS defaultCRS;

  private List<CRS> supportedCRSs;

  public DefaultCRSRegistry() {
    this.defaultCRS = new DefaultCRS("urn:ogc:def:crs:EPSG::4326"); //$NON-NLS-1$
    this.supportedCRSs = new LinkedList<CRS>();
    this.supportedCRSs.add(defaultCRS);
  }

  @Override
  public CRS getDefaultCRS() {
    return defaultCRS;
  }

  public void setDefaultCRS(CRS defaultCRS) {
    notNull(defaultCRS);
    this.defaultCRS = defaultCRS;
  }

  @Override
  public List<CRS> getSupportedCRSs() {
    return supportedCRSs;
  }

  public void addSupportedCRS(CRS supportedCRS) {
    notNull(supportedCRS);
    this.supportedCRSs.add(supportedCRS);
  }

}
