/**
 * Legato is a configurable, lightweight web mapping client that can be
 * easily embedded into web pages and portals, CMS and individual web
 * applications. Legato is implemented in JavaScript and based on the
 * popular open source library OpenLayers.
 *
 * Copyright (C) 2010  disy Informationssysteme GmbH, http://www.disy.net
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package net.disy.ogc.wps.v_1_0_0.model;

import org.apache.commons.lang.Validate;

public class DefaultDataType<T> implements DataType<T> {


  private final String schemaDesignator;

  private final Class<T> targetClass;

  private final DataTypeType type;

  public DefaultDataType(
      String schemaDesignator,
      Class<T> target,
      DataTypeType type) {
    Validate.notNull(schemaDesignator);
    Validate.notNull(target);
    Validate.notNull(type);
    this.schemaDesignator = schemaDesignator;
    this.targetClass = target;
    this.type = type;
  }

  @Override
  public String getSchemaDesignator() {
    return schemaDesignator;
  }

  @Override
  public Class<T> getTargetClass() {
    return targetClass;
  }

  @Override
  public int hashCode() {
    final int prime = 31;
    int result = 1;
    result = prime * result + ((schemaDesignator == null) ? 0 : schemaDesignator.hashCode());
    return result;
  }

  @Override
  public boolean equals(Object obj) {
    if (this == obj) {
      return true;
    }
    if (obj == null) {
      return false;
    }
    if (getClass() != obj.getClass()) {
      return false;
    }
    final DefaultDataType<?> other = (DefaultDataType<?>) obj;
    if (schemaDesignator == null) {
      if (other.schemaDesignator != null) {
        return false;
      }
    }
    else if (!schemaDesignator.equals(other.schemaDesignator)) {
      return false;
    }
    return true;
  }

  @Override
  public DataTypeType getType() {
    return type;
  }
}
