/**
 * Legato is a configurable, lightweight web mapping client that can be
 * easily embedded into web pages and portals, CMS and individual web
 * applications. Legato is implemented in JavaScript and based on the
 * popular open source library OpenLayers.
 *
 * Copyright (C) 2010  disy Informationssysteme GmbH, http://www.disy.net
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package net.disy.ogc.wps.v_1_0_0.model;

import static net.disy.ogc.wps.v_1_0_0.util.Validate.*;
import static org.apache.commons.lang.Validate.*;

import java.util.HashMap;
import java.util.Map;

public class DefaultDataTypeRegistry implements DataTypeRegistry {
  private final Map<Class<?>, DataType<?>> dataTypesByDefaultClass = new HashMap<Class<?>, DataType<?>>();
  private final Map<String, DataType<?>> dataTypesBySchemaDesignator = new HashMap<String, DataType<?>>();

  @Override
  public DataType<?> getDataType(String schemaDesignator) throws IllegalArgumentException {
    notNull(schemaDesignator);
    contains(dataTypesBySchemaDesignator, schemaDesignator, "No dataType for schemaDesignator '" //$NON-NLS-1$
        + schemaDesignator
        + "'"); //$NON-NLS-1$
    return dataTypesBySchemaDesignator.get(schemaDesignator);
  }

  @Override
  public DataType<?> getDefaultDataType(Class<?> typeClass) throws IllegalArgumentException {
    notNull(typeClass);
    contains(
        dataTypesByDefaultClass,
        typeClass,
        "No dataType for class '" + typeClass.getName() + "'"); //$NON-NLS-1$ //$NON-NLS-2$
    return dataTypesByDefaultClass.get(typeClass);
  }

  public void addDataTypeForDefaultClass(Class<?> typeClass, DataType<?> dataType) {
    notNull(typeClass, dataType);
    notContains(dataTypesByDefaultClass, typeClass);
    dataTypesByDefaultClass.put(typeClass, dataType);
  }

  public void addDataTypeForSchemaDesignator(DataType<?> dataType) {
    notNull(dataType);
    notContains(dataTypesBySchemaDesignator, dataType.getSchemaDesignator());
    dataTypesBySchemaDesignator.put(dataType.getSchemaDesignator(), dataType);
  }
}
