/**
 * Legato is a configurable, lightweight web mapping client that can be
 * easily embedded into web pages and portals, CMS and individual web
 * applications. Legato is implemented in JavaScript and based on the
 * popular open source library OpenLayers.
 *
 * Copyright (C) 2010  disy Informationssysteme GmbH, http://www.disy.net
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package net.disy.ogc.wps.v_1_0_0.model;

import static net.disy.ogc.wps.v_1_0_0.util.Validate.notContains;
import static org.apache.commons.lang.Validate.notNull;

import java.util.HashMap;
import java.util.Map;

import net.disy.ogc.wps.v_1_0_0.converter.SingleItemTypeConverter;

import org.apache.commons.lang.Validate;

public class DefaultFormatRegistry implements FormatRegistry {
  private final Map<DataType<?>, Format<?>> defaultFormatsByDataType = new HashMap<DataType<?>, Format<?>>();
  private final Map<FormatId, Format<?>> formatsById = new HashMap<FormatId, Format<?>>();

  @Override
  public <T> Format<T> getDefaultFormat(DataType<T> dataType) throws IllegalArgumentException {
    Validate.notNull(dataType);
    if (defaultFormatsByDataType.containsKey(dataType)) {
      @SuppressWarnings("unchecked")
      final Format<T> format = (Format<T>) defaultFormatsByDataType.get(dataType);
      return format;
    }
    else {
      return new DefaultFormat<T>(
          dataType,
          new FormatId(
              XmlSchemaConstants.TEXT_XML_MIME_TYPE,
              XmlSchemaConstants.UTF8_ENCODING,
              dataType.getSchemaDesignator()), new SingleItemTypeConverter<T>(
              dataType.getTargetClass()));
    }
  }

  public <T> void addDefaultFormat(Format<?> format) {
    notNull(format);
    notContains(defaultFormatsByDataType, format.getDataType());
    defaultFormatsByDataType.put(format.getDataType(), format);
  }

  public void addFormat(Format<?> format) {
    notNull(format);
    notContains(formatsById, format.getFormatId());
    formatsById.put(format.getFormatId(), format);
  }

  //  @Override
  //  public <T> Collection<? extends Format<T>> getFormats(DataType<T> dataType) {
  //    // TODO Auto-generated method stub
  //    return null;
  //  }

}
