/**
 * Legato is a configurable, lightweight web mapping client that can be
 * easily embedded into web pages and portals, CMS and individual web
 * applications. Legato is implemented in JavaScript and based on the
 * popular open source library OpenLayers.
 *
 * Copyright (C) 2010  disy Informationssysteme GmbH, http://www.disy.net
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package net.disy.ogc.wps.v_1_0_0.model;

import static org.apache.commons.lang.Validate.*;

import java.text.MessageFormat;

public class FormatId {
  private final String mimeType;
  private final String encoding;
  private final String schemaDesignator;

  public FormatId(String mimeType, String encoding, String schemaDesignator) {
    notNull(mimeType);
    notNull(encoding);
    notNull(schemaDesignator);
    this.mimeType = mimeType;
    this.encoding = encoding;
    this.schemaDesignator = schemaDesignator;
  }

  public String getMimeType() {
    return mimeType;
  }

  public String getEncoding() {
    return encoding;
  }

  public String getSchemaDesignator() {
    return schemaDesignator;
  }

  @Override
  public int hashCode() {
    return mimeType.hashCode() + encoding.hashCode() + schemaDesignator.hashCode();
  }

  @Override
  public boolean equals(Object o) {
    if (!(o instanceof FormatId)) {
      return false;
    }

    FormatId other = (FormatId) o;
    return mimeType.equals(other.mimeType)
        && encoding.equals(other.encoding)
        && schemaDesignator.equals(other.schemaDesignator);
  }

  @Override
  public String toString() {
    return MessageFormat.format(
        "FormatId [mimeType={0}, encoding={1}, schemaDesignator={2}]",
        mimeType,
        encoding,
        schemaDesignator);
  }

}
