/**
 * Legato is a configurable, lightweight web mapping client that can be
 * easily embedded into web pages and portals, CMS and individual web
 * applications. Legato is implemented in JavaScript and based on the
 * popular open source library OpenLayers.
 *
 * Copyright (C) 2010  disy Informationssysteme GmbH, http://www.disy.net
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package net.disy.ogc.wps.v_1_0_0.model;

import javax.xml.namespace.QName;

import org.apache.commons.lang.Validate;

public class SchemaReference {

  public static char DELIMITER = '#';

  public static String TYPE_PREFIX = "~"; //$NON-NLS-1$
  public static String ELEMENT_PREFIX = ""; //$NON-NLS-1$

  private final String location;

  private final String namespaceUri;

  private final String defaultPrefix;

  public SchemaReference(String location, String namespaceUri) {
    this(location, namespaceUri, null);
  }

  public SchemaReference(String location, String namespaceUri, String defaultPrefix) {
    super();
    Validate.notNull(location);
    Validate.notNull(namespaceUri);
    this.location = location;
    this.namespaceUri = namespaceUri;
    this.defaultPrefix = defaultPrefix;
  }

  public QName qname(String localPart) {
    if (this.defaultPrefix == null) {
      return new QName(this.namespaceUri, localPart);
    }
    else {
      return new QName(this.namespaceUri, localPart, this.defaultPrefix);
    }
  }

  public String typeDesignator(String localTypeName) {
    Validate.notNull(localTypeName);
    return this.location + DELIMITER + TYPE_PREFIX + localTypeName;

  }

  public String elementDesignator(String localElementName) {
    Validate.notNull(localElementName);
    return this.location + DELIMITER + ELEMENT_PREFIX + localElementName;

  }

  public <T> DataType<T> literalDataType(String localTypeName, Class<T> defaultClass) {
    return new DefaultDataType<T>(
        typeDesignator(localTypeName),
        defaultClass,
        DataTypeType.LITERAL);

  }

  public <T> DataType<T> bboxDataType(String localElementName, Class<T> defaultClass) {
    return new DefaultDataType<T>(
        elementDesignator(localElementName),
        defaultClass,
        DataTypeType.BBOX);

  }

  public <T> DataType<T> complexDataType(String localElementName, Class<T> defaultClass) {
    return new DefaultDataType<T>(
        elementDesignator(localElementName),
        defaultClass,
        DataTypeType.COMPLEX);

  }
}
