/**
 * Legato is a configurable, lightweight web mapping client that can be
 * easily embedded into web pages and portals, CMS and individual web
 * applications. Legato is implemented in JavaScript and based on the
 * popular open source library OpenLayers.
 *
 * Copyright (C) 2010  disy Informationssysteme GmbH, http://www.disy.net
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package net.disy.ogc.wps.v_1_0_0.model;

import javax.xml.datatype.XMLGregorianCalendar;
import javax.xml.namespace.QName;

import net.disy.ogc.gml.v_3_1_1.GmlConstants;
import net.disy.ogc.ows.v_1_1_0.OwsConstants;
import net.disy.ogc.wps.v_1_0_0.WpsConstants;
import net.disy.ogc.wps.v_1_0_0.WpsSchemaConstants;
import net.disy.ogc.wps.v_1_0_0.converter.ConcatenatedConverter;
import net.disy.ogc.wps.v_1_0_0.converter.DefaultJAXBElementTypeConverter;
import net.disy.ogc.wps.v_1_0_0.converter.DefaultTypeConverterRegistry;
import net.disy.ogc.wps.v_1_0_0.converter.DoubleFloatConverter;
import net.disy.ogc.wps.v_1_0_0.converter.GeometryConverter;
import net.disy.ogc.wps.v_1_0_0.converter.LongByteConverter;
import net.disy.ogc.wps.v_1_0_0.converter.LongIntegerConverter;
import net.disy.ogc.wps.v_1_0_0.converter.LongShortConverter;
import net.disy.ogc.wps.v_1_0_0.converter.ObjectElementConverter;
import net.disy.ogc.wps.v_1_0_0.converter.PrimitiveTypeConverter;
import net.disy.ogc.wps.v_1_0_0.converter.SingleItemTypeConverter;
import net.disy.ogc.wps.v_1_0_0.converter.SuperclassingConverter;
import net.disy.ogc.wps.v_1_0_0.procedure.DefaultWpsProcessContext;
import net.disy.ogc.wpspd.v_1_0_0.Group;
import net.disy.ogc.wpspd.v_1_0_0.Link;
import net.disy.ogc.wpspd.v_1_0_0.Marker;
import net.disy.ogc.wpspd.v_1_0_0.Message;
import net.disy.ogc.wpspd.v_1_0_0.Viewport;
import net.disy.ogc.wpspd.v_1_0_0.WpspdSchemaConstants;
import net.opengis.ows.v_1_1_0.BoundingBoxType;
import net.opengis.wps.v_1_0_0.ProcessDescriptionType;

import org.w3c.dom.Element;

import com.vividsolutions.jts.geom.Geometry;
import com.vividsolutions.jts.geom.GeometryCollection;
import com.vividsolutions.jts.geom.LineString;
import com.vividsolutions.jts.geom.MultiLineString;
import com.vividsolutions.jts.geom.MultiPoint;
import com.vividsolutions.jts.geom.MultiPolygon;
import com.vividsolutions.jts.geom.Point;
import com.vividsolutions.jts.geom.Polygon;

public class WpsProcessConfigurator {

  public void configure(DefaultWpsProcessContext wpsProcessContext) {
    configure(wpsProcessContext.getDataTypeRegistry());
    configure(wpsProcessContext, wpsProcessContext.getConverterRegistry());
    configure(wpsProcessContext.getLiteralTypeRegistry());
    configure(wpsProcessContext.getFormatRegistry());
    configure(wpsProcessContext.getCRSRegistry());
  }

  public void configure(DefaultDataTypeRegistry dataTypeRegistry) {
    // XSD
    dataTypeRegistry
        .addDataTypeForDefaultClass(Object.class, XmlSchemaConstants.ANY_TYPE_DATA_TYPE);
    dataTypeRegistry.addDataTypeForDefaultClass(
        Element.class,
        XmlSchemaConstants.ANY_TYPE_DATA_TYPE);
    dataTypeRegistry.addDataTypeForSchemaDesignator(XmlSchemaConstants.ANY_TYPE_DATA_TYPE);

    dataTypeRegistry.addDataTypeForDefaultClass(String.class, XmlSchemaConstants.STRING_DATA_TYPE);
    dataTypeRegistry.addDataTypeForSchemaDesignator(XmlSchemaConstants.STRING_DATA_TYPE);

    dataTypeRegistry.addDataTypeForDefaultClass(Long.class, XmlSchemaConstants.LONG_DATA_TYPE);
    dataTypeRegistry.addDataTypeForDefaultClass(long.class, XmlSchemaConstants.LONG_DATA_TYPE);
    dataTypeRegistry.addDataTypeForDefaultClass(Integer.class, XmlSchemaConstants.LONG_DATA_TYPE);
    dataTypeRegistry.addDataTypeForDefaultClass(int.class, XmlSchemaConstants.LONG_DATA_TYPE);
    dataTypeRegistry.addDataTypeForDefaultClass(Short.class, XmlSchemaConstants.LONG_DATA_TYPE);
    dataTypeRegistry.addDataTypeForDefaultClass(short.class, XmlSchemaConstants.LONG_DATA_TYPE);
    dataTypeRegistry.addDataTypeForDefaultClass(Byte.class, XmlSchemaConstants.LONG_DATA_TYPE);
    dataTypeRegistry.addDataTypeForDefaultClass(byte.class, XmlSchemaConstants.LONG_DATA_TYPE);
    dataTypeRegistry.addDataTypeForSchemaDesignator(XmlSchemaConstants.LONG_DATA_TYPE);

    dataTypeRegistry.addDataTypeForDefaultClass(Double.class, XmlSchemaConstants.DOUBLE_DATA_TYPE);
    dataTypeRegistry.addDataTypeForDefaultClass(double.class, XmlSchemaConstants.DOUBLE_DATA_TYPE);
    dataTypeRegistry.addDataTypeForDefaultClass(Float.class, XmlSchemaConstants.LONG_DATA_TYPE);
    dataTypeRegistry.addDataTypeForDefaultClass(float.class, XmlSchemaConstants.LONG_DATA_TYPE);
    dataTypeRegistry.addDataTypeForSchemaDesignator(XmlSchemaConstants.DOUBLE_DATA_TYPE);

    dataTypeRegistry
        .addDataTypeForDefaultClass(Boolean.class, XmlSchemaConstants.BOOLEAN_DATA_TYPE);
    dataTypeRegistry
        .addDataTypeForDefaultClass(boolean.class, XmlSchemaConstants.BOOLEAN_DATA_TYPE);
    dataTypeRegistry.addDataTypeForSchemaDesignator(XmlSchemaConstants.BOOLEAN_DATA_TYPE);

    dataTypeRegistry.addDataTypeForDefaultClass(
        XMLGregorianCalendar.class,
        XmlSchemaConstants.DATE_DATA_TYPE);
    dataTypeRegistry.addDataTypeForSchemaDesignator(XmlSchemaConstants.DATE_DATA_TYPE);

    // OWS

    dataTypeRegistry.addDataTypeForDefaultClass(
        BoundingBoxType.class,
        OwsConstants.BOUNDING_BOX_DATA_TYPE);
    dataTypeRegistry.addDataTypeForSchemaDesignator(OwsConstants.BOUNDING_BOX_DATA_TYPE);

    // GML
    dataTypeRegistry.addDataTypeForDefaultClass(Point.class, GmlConstants.POINT_DATA_TYPE);
    dataTypeRegistry.addDataTypeForSchemaDesignator(GmlConstants.POINT_DATA_TYPE);

    dataTypeRegistry.addDataTypeForDefaultClass(
        LineString.class,
        GmlConstants.LINE_STRING_DATA_TYPE);
    dataTypeRegistry.addDataTypeForSchemaDesignator(GmlConstants.LINE_STRING_DATA_TYPE);

    dataTypeRegistry.addDataTypeForDefaultClass(Polygon.class, GmlConstants.POLYGON_DATA_TYPE);
    dataTypeRegistry.addDataTypeForSchemaDesignator(GmlConstants.POLYGON_DATA_TYPE);

    dataTypeRegistry.addDataTypeForDefaultClass(Geometry.class, GmlConstants.GEOMETRY_DATA_TYPE);
    dataTypeRegistry.addDataTypeForSchemaDesignator(GmlConstants.GEOMETRY_DATA_TYPE);

    dataTypeRegistry.addDataTypeForDefaultClass(
        MultiPoint.class,
        GmlConstants.MULTI_POINT_DATA_TYPE);
    dataTypeRegistry.addDataTypeForSchemaDesignator(GmlConstants.MULTI_POINT_DATA_TYPE);

    dataTypeRegistry.addDataTypeForDefaultClass(
        MultiLineString.class,
        GmlConstants.MULTI_LINE_STRING_DATA_TYPE);
    dataTypeRegistry.addDataTypeForSchemaDesignator(GmlConstants.MULTI_LINE_STRING_DATA_TYPE);

    dataTypeRegistry.addDataTypeForDefaultClass(
        MultiPolygon.class,
        GmlConstants.MULTI_POLYGON_DATA_TYPE);
    dataTypeRegistry.addDataTypeForSchemaDesignator(GmlConstants.MULTI_POLYGON_DATA_TYPE);

    dataTypeRegistry.addDataTypeForDefaultClass(
        GeometryCollection.class,
        GmlConstants.MULTI_GEOMETRY_DATA_TYPE);
    dataTypeRegistry.addDataTypeForSchemaDesignator(GmlConstants.MULTI_GEOMETRY_DATA_TYPE);

    // WPSPD
    {
      {
        dataTypeRegistry
            .addDataTypeForDefaultClass(Link.class, WpspdSchemaConstants.LINK_DATA_TYPE);
        dataTypeRegistry.addDataTypeForSchemaDesignator(WpspdSchemaConstants.LINK_DATA_TYPE);
      }

      {
        dataTypeRegistry.addDataTypeForDefaultClass(
            Message.class,
            WpspdSchemaConstants.MESSAGE_DATA_TYPE);
        dataTypeRegistry.addDataTypeForSchemaDesignator(WpspdSchemaConstants.MESSAGE_DATA_TYPE);
      }

      {
        dataTypeRegistry.addDataTypeForDefaultClass(
            Viewport.class,
            WpspdSchemaConstants.VIEWPORT_DATA_TYPE);
        dataTypeRegistry.addDataTypeForSchemaDesignator(WpspdSchemaConstants.VIEWPORT_DATA_TYPE);
      }

      {
        dataTypeRegistry.addDataTypeForDefaultClass(
            Marker.class,
            WpspdSchemaConstants.MARKER_DATA_TYPE);
        dataTypeRegistry.addDataTypeForSchemaDesignator(WpspdSchemaConstants.MARKER_DATA_TYPE);
      }

      {
        dataTypeRegistry.addDataTypeForDefaultClass(
            net.disy.ogc.wpspd.v_1_0_0.Geometry.class,
            WpspdSchemaConstants.GEOMETRY_DATA_TYPE);
        dataTypeRegistry.addDataTypeForSchemaDesignator(WpspdSchemaConstants.GEOMETRY_DATA_TYPE);
      }

      {
        dataTypeRegistry.addDataTypeForDefaultClass(
            Group.class,
            WpspdSchemaConstants.GROUP_DATA_TYPE);
        dataTypeRegistry.addDataTypeForSchemaDesignator(WpspdSchemaConstants.GROUP_DATA_TYPE);
      }
    }
    // WPS
    {
      {
        dataTypeRegistry.addDataTypeForDefaultClass(
            ProcessDescriptionType.class,
            WpsSchemaConstants.PROCESSDESCRIPTION_DATA_TYPE);
        dataTypeRegistry
            .addDataTypeForSchemaDesignator(WpsSchemaConstants.PROCESSDESCRIPTION_DATA_TYPE);
      }
    }
  }

  public void configure(DefaultLiteralTypeRegistry literalTypeRegistry) {
    literalTypeRegistry.add(XmlSchemaConstants.STRING_LITERAL_TYPE);
    literalTypeRegistry.add(XmlSchemaConstants.LONG_LITERAL_TYPE);
    literalTypeRegistry.add(XmlSchemaConstants.DOUBLE_LITERAL_TYPE);
    literalTypeRegistry.add(XmlSchemaConstants.BOOLEAN_LITERAL_TYPE);
    literalTypeRegistry.add(XmlSchemaConstants.DATE_LITERAL_TYPE);
  }

  public void configure(DefaultCRSRegistry registry) {
    //    registry.setDefaultCRS(defaultCRS)
    // TODO Auto-generated method stub

  }

  public void configure(
      DefaultWpsProcessContext wpsProcessContext,
      DefaultTypeConverterRegistry converterRegistry) {

    converterRegistry.addConverter(new ObjectElementConverter(wpsProcessContext.getJAXBContext()));

    converterRegistry.addConverter(new PrimitiveTypeConverter<Long>(long.class));
    converterRegistry.addConverter(new PrimitiveTypeConverter<Double>(double.class));
    converterRegistry.addConverter(new PrimitiveTypeConverter<Boolean>(boolean.class));

    LongIntegerConverter longIntegerConverter = new LongIntegerConverter();
    converterRegistry.addConverter(longIntegerConverter);
    converterRegistry.addConverter(new ConcatenatedConverter<Long, Integer, Integer>(
        longIntegerConverter,
        new PrimitiveTypeConverter<Integer>(int.class)));

    LongShortConverter longShortConverter = new LongShortConverter();
    converterRegistry.addConverter(longShortConverter);
    converterRegistry.addConverter(new ConcatenatedConverter<Long, Short, Short>(
        longShortConverter,
        new PrimitiveTypeConverter<Short>(short.class)));

    LongByteConverter longByteConverter = new LongByteConverter();
    converterRegistry.addConverter(longByteConverter);
    converterRegistry.addConverter(new ConcatenatedConverter<Long, Byte, Byte>(
        longByteConverter,
        new PrimitiveTypeConverter<Byte>(byte.class)));

    DoubleFloatConverter doubleFloatConverter = new DoubleFloatConverter();
    converterRegistry.addConverter(doubleFloatConverter);
    converterRegistry.addConverter(new ConcatenatedConverter<Double, Float, Float>(
        doubleFloatConverter,
        new PrimitiveTypeConverter<Float>(float.class)));

    // ???????????????????????????????????????????????
    converterRegistry.addConverter(new SuperclassingConverter<Geometry, Polygon>(
        Geometry.class,
        Polygon.class));

  }

  public void configure(DefaultFormatRegistry formatRegistry) {
    // XSD
    {
      Format<Object> format = new DefaultFormat<Object>(
          XmlSchemaConstants.ANY_TYPE_DATA_TYPE,
          new FormatId(
              XmlSchemaConstants.TEXT_XML_MIME_TYPE,
              XmlSchemaConstants.UTF8_ENCODING,
              XmlSchemaConstants.ANY_TYPE_SCHEMA_DESIGNATOR), new SingleItemTypeConverter<Object>(
              Object.class));
      formatRegistry.addDefaultFormat(format);
      formatRegistry.addFormat(format);
    }
    // GML
    {
      Format<Point> format = createGeometryFormat(
          GmlConstants.POINT_DATA_TYPE,
          GmlConstants.POINT_SCHEMA_DESIGNATOR,
          Point.class);
      formatRegistry.addDefaultFormat(format);
      formatRegistry.addFormat(format);
    }
    {
      Format<LineString> format = createGeometryFormat(
          GmlConstants.LINE_STRING_DATA_TYPE,
          GmlConstants.LINE_STRING_SCHEMA_DESIGNATOR,
          LineString.class);
      formatRegistry.addDefaultFormat(format);
      formatRegistry.addFormat(format);
    }
    {
      Format<Polygon> format = createGeometryFormat(
          GmlConstants.POLYGON_DATA_TYPE,
          GmlConstants.POLYGON_SCHEMA_DESIGNATOR,
          Polygon.class);
      formatRegistry.addDefaultFormat(format);
      formatRegistry.addFormat(format);
    }
    {

      Format<Geometry> format = createGeometryFormat(
          GmlConstants.GEOMETRY_DATA_TYPE,
          GmlConstants.GEOMETRY_SCHEMA_DESIGNATOR,
          Geometry.class);
      formatRegistry.addDefaultFormat(format);
      formatRegistry.addFormat(format);
    }
    {
      Format<MultiPoint> format = createGeometryFormat(
          GmlConstants.MULTI_POINT_DATA_TYPE,
          GmlConstants.MULTI_POINT_SCHEMA_DESIGNATOR,
          MultiPoint.class);
      formatRegistry.addDefaultFormat(format);
      formatRegistry.addFormat(format);
    }
    {
      Format<MultiLineString> format = createGeometryFormat(
          GmlConstants.MULTI_LINE_STRING_DATA_TYPE,
          GmlConstants.MULTI_LINE_STRING_SCHEMA_DESIGNATOR,
          MultiLineString.class);
      formatRegistry.addDefaultFormat(format);
      formatRegistry.addFormat(format);
    }
    {
      Format<MultiPolygon> format = createGeometryFormat(
          GmlConstants.MULTI_POLYGON_DATA_TYPE,
          GmlConstants.MULTI_POLYGON_SCHEMA_DESIGNATOR,
          MultiPolygon.class);
      formatRegistry.addDefaultFormat(format);
      formatRegistry.addFormat(format);
    }
    {

      Format<GeometryCollection> format = createGeometryFormat(
          GmlConstants.MULTI_GEOMETRY_DATA_TYPE,
          GmlConstants.MULTI_GEOMETRY_SCHEMA_DESIGNATOR,
          GeometryCollection.class);
      formatRegistry.addDefaultFormat(format);
      formatRegistry.addFormat(format);
    }
    // WPSPD
    {
      Format<Link> format =

      new DefaultFormat<Link>(WpspdSchemaConstants.LINK_DATA_TYPE, new FormatId(
          XmlSchemaConstants.TEXT_XML_MIME_TYPE,
          XmlSchemaConstants.UTF8_ENCODING,
          WpspdSchemaConstants.LINK_SCHEMA_DESIGNATOR), new SingleItemTypeConverter<Link>(
          Link.class));

      formatRegistry.addDefaultFormat(format);
      formatRegistry.addFormat(format);
    }
    {
      Format<Message> format =

      new DefaultFormat<Message>(WpspdSchemaConstants.MESSAGE_DATA_TYPE, new FormatId(
          XmlSchemaConstants.TEXT_XML_MIME_TYPE,
          XmlSchemaConstants.UTF8_ENCODING,
          WpspdSchemaConstants.MESSAGE_SCHEMA_DESIGNATOR), new SingleItemTypeConverter<Message>(
          Message.class));

      formatRegistry.addDefaultFormat(format);
      formatRegistry.addFormat(format);
    }
    {
      Format<Viewport> format =

      new DefaultFormat<Viewport>(WpspdSchemaConstants.VIEWPORT_DATA_TYPE, new FormatId(
          XmlSchemaConstants.TEXT_XML_MIME_TYPE,
          XmlSchemaConstants.UTF8_ENCODING,
          WpspdSchemaConstants.VIEWPORT_SCHEMA_DESIGNATOR), new SingleItemTypeConverter<Viewport>(
          Viewport.class));

      formatRegistry.addDefaultFormat(format);
      formatRegistry.addFormat(format);
    }
    {
      Format<Marker> format =

      new DefaultFormat<Marker>(WpspdSchemaConstants.MARKER_DATA_TYPE, new FormatId(
          XmlSchemaConstants.TEXT_XML_MIME_TYPE,
          XmlSchemaConstants.UTF8_ENCODING,
          WpspdSchemaConstants.MARKER_SCHEMA_DESIGNATOR), new SingleItemTypeConverter<Marker>(
          Marker.class));

      formatRegistry.addDefaultFormat(format);
      formatRegistry.addFormat(format);
      // marker
    }
    {
      Format<net.disy.ogc.wpspd.v_1_0_0.Geometry> format =

      new DefaultFormat<net.disy.ogc.wpspd.v_1_0_0.Geometry>(
          WpspdSchemaConstants.GEOMETRY_DATA_TYPE,
          new FormatId(
              XmlSchemaConstants.TEXT_XML_MIME_TYPE,
              XmlSchemaConstants.UTF8_ENCODING,
              WpspdSchemaConstants.GEOMETRY_SCHEMA_DESIGNATOR),
          new SingleItemTypeConverter<net.disy.ogc.wpspd.v_1_0_0.Geometry>(
              net.disy.ogc.wpspd.v_1_0_0.Geometry.class));

      formatRegistry.addDefaultFormat(format);
      formatRegistry.addFormat(format);
      //geom
    }

    {
      Format<Group> groupFormat = new DefaultFormat<Group>(
          WpspdSchemaConstants.GROUP_DATA_TYPE,
          new FormatId(
              XmlSchemaConstants.TEXT_XML_MIME_TYPE,
              XmlSchemaConstants.UTF8_ENCODING,
              WpspdSchemaConstants.GROUP_SCHEMA_DESIGNATOR), new SingleItemTypeConverter<Group>(
              Group.class));

      formatRegistry.addDefaultFormat(groupFormat);
      formatRegistry.addFormat(groupFormat);
    }
    // WPS
    Format<ProcessDescriptionType> processDescriptionFormat =

    new DefaultFormat<ProcessDescriptionType>(
        WpsSchemaConstants.PROCESSDESCRIPTION_DATA_TYPE,
        new FormatId(
            XmlSchemaConstants.TEXT_XML_MIME_TYPE,
            XmlSchemaConstants.UTF8_ENCODING,
            WpsSchemaConstants.PROCESSDESCRIPTION_SCHEMA_DESIGNATOR),
        new DefaultJAXBElementTypeConverter<ProcessDescriptionType>(
            ProcessDescriptionType.class,
            new QName(
                WpsConstants.NAMESPACE_URI,
                WpsSchemaConstants.PROCESSDESCRIPTION_LOCAL_NAME,
                WpsConstants.DEFAULT_PREFIX)));

    formatRegistry.addDefaultFormat(processDescriptionFormat);
    formatRegistry.addFormat(processDescriptionFormat);
  }

  private <T extends Geometry> Format<T> createGeometryFormat(
      DataType<T> dataType,
      String schemaDesignator,
      Class<T> geometryClass) {
    FormatId formatId = new FormatId(
        XmlSchemaConstants.TEXT_XML_MIME_TYPE,
        XmlSchemaConstants.UTF8_ENCODING,
        schemaDesignator);
    return new DefaultFormat<T>(dataType, formatId, new GeometryConverter<T>(geometryClass));
  }
}
