/**
 * Legato is a configurable, lightweight web mapping client that can be
 * easily embedded into web pages and portals, CMS and individual web
 * applications. Legato is implemented in JavaScript and based on the
 * popular open source library OpenLayers.
 *
 * Copyright (C) 2010  disy Informationssysteme GmbH, http://www.disy.net
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package net.disy.ogc.wps.v_1_0_0.model;

import java.net.URI;

import javax.xml.datatype.XMLGregorianCalendar;
import javax.xml.namespace.QName;

import net.disy.ogc.wps.v_1_0_0.converter.IdenticalConverter;
import net.disy.ogc.wps.v_1_0_0.converter.SingleItemTypeConverter;
import net.disy.ogc.wps.v_1_0_0.converter.StringBooleanConverter;
import net.disy.ogc.wps.v_1_0_0.converter.StringCalendarConverter;
import net.disy.ogc.wps.v_1_0_0.converter.StringDoubleConverter;
import net.disy.ogc.wps.v_1_0_0.converter.StringLongConverter;

public class XmlSchemaConstants {

  private XmlSchemaConstants() {
    // nothing to do
  }

  public final static String UTF8_ENCODING = "utf-8"; //$NON-NLS-1$
  public final static String TEXT_XML_MIME_TYPE = "text/xml"; //$NON-NLS-1$

  public final static String LOCATION = "http://www.w3.org/2001/XMLSchema.xsd"; //$NON-NLS-1$
  public final static String NAMESPACE_URI = "http://www.w3.org/2001/XMLSchema"; //$NON-NLS-1$

  public final static SchemaReference XSD = new SchemaReference(LOCATION, NAMESPACE_URI, "xsd"); //$NON-NLS-1$

  public final static String ANY_TYPE_LOCAL_NAME = "anyType"; //$NON-NLS-1$
  public final static String STRING_LOCAL_NAME = "string"; //$NON-NLS-1$
  public final static String LONG_LOCAL_NAME = "long"; //$NON-NLS-1$
  public final static String DOUBLE_LOCAL_NAME = "double"; //$NON-NLS-1$
  public final static String BOOLEAN_LOCAL_NAME = "boolean"; //$NON-NLS-1$
  public final static String DATE_LOCAL_NAME = "date"; //$NON-NLS-1$
  public final static String ANY_URI_LOCAL_NAME = "anyURI"; //$NON-NLS-1$

  public final static String ANY_TYPE_SCHEMA_DESIGNATOR = XSD.typeDesignator(ANY_TYPE_LOCAL_NAME);
  public final static String STRING_SCHEMA_DESIGNATOR = XSD.typeDesignator(STRING_LOCAL_NAME);
  public final static String LONG_SCHEMA_DESIGNATOR = XSD.typeDesignator(LONG_LOCAL_NAME);
  public final static String DOUBLE_SCHEMA_DESIGNATOR = XSD.typeDesignator(DOUBLE_LOCAL_NAME);
  public final static String BOOLEAN_SCHEMA_DESIGNATOR = XSD.typeDesignator(BOOLEAN_LOCAL_NAME);
  public final static String DATE_SCHEMA_DESIGNATOR = XSD.typeDesignator(DATE_LOCAL_NAME);
  public final static String ANYURI_SCHEMA_DESIGNATOR = XSD.typeDesignator(ANY_URI_LOCAL_NAME);

  public final static QName STRING_TYPE_NAME = XSD.qname(STRING_LOCAL_NAME);
  public final static QName LONG_TYPE_NAME = XSD.qname(LONG_LOCAL_NAME);
  public final static QName DOUBLE_TYPE_NAME = XSD.qname(DOUBLE_LOCAL_NAME);
  public final static QName BOOLEAN_TYPE_NAME = XSD.qname(BOOLEAN_LOCAL_NAME);
  public final static QName DATE_TYPE_NAME = XSD.qname(DATE_LOCAL_NAME);
  public final static QName ANYURI_TYPE_NAME = XSD.qname(ANY_URI_LOCAL_NAME);

  public final static DataType<Object> ANY_TYPE_DATA_TYPE = new DefaultDataType<Object>(
      ANY_TYPE_SCHEMA_DESIGNATOR,
      Object.class,
      DataTypeType.COMPLEX);

  public final static DataType<String> STRING_DATA_TYPE = XSD.literalDataType(
      STRING_LOCAL_NAME,
      String.class);
  public final static DataType<Long> LONG_DATA_TYPE = XSD.literalDataType(
      LONG_LOCAL_NAME,
      Long.class);
  public final static DataType<Double> DOUBLE_DATA_TYPE = XSD.literalDataType(
      DOUBLE_LOCAL_NAME,
      Double.class);
  public final static DataType<Boolean> BOOLEAN_DATA_TYPE = XSD.literalDataType(
      BOOLEAN_LOCAL_NAME,
      Boolean.class);
  public final static DataType<XMLGregorianCalendar> DATE_DATA_TYPE = XSD.literalDataType(
      DATE_LOCAL_NAME,
      XMLGregorianCalendar.class);
  public final static DataType<URI> ANYURI_DATA_TYPE = XSD.literalDataType(
      ANY_URI_LOCAL_NAME,
      URI.class);

  public final static LiteralType<String> STRING_LITERAL_TYPE = new DefaultLiteralType<String>(
      STRING_DATA_TYPE,
      new IdenticalConverter<String>(String.class));
  public final static LiteralType<Long> LONG_LITERAL_TYPE = new DefaultLiteralType<Long>(
      LONG_DATA_TYPE,
      new StringLongConverter());
  public final static LiteralType<Double> DOUBLE_LITERAL_TYPE = new DefaultLiteralType<Double>(
      DOUBLE_DATA_TYPE,
      new StringDoubleConverter());
  public final static LiteralType<Boolean> BOOLEAN_LITERAL_TYPE = new DefaultLiteralType<Boolean>(
      BOOLEAN_DATA_TYPE,
      new StringBooleanConverter());
  public final static LiteralType<XMLGregorianCalendar> DATE_LITERAL_TYPE = new DefaultLiteralType<XMLGregorianCalendar>(
      DATE_DATA_TYPE,
      new StringCalendarConverter());

  public static final FormatId ANY_TYPE_FORMAT_ID = new FormatId(
      XmlSchemaConstants.TEXT_XML_MIME_TYPE,
      XmlSchemaConstants.UTF8_ENCODING,
      XmlSchemaConstants.ANY_TYPE_SCHEMA_DESIGNATOR);

  public static final Format<Object> ANY_TYPE_FORMAT = new DefaultFormat<Object>(
      XmlSchemaConstants.ANY_TYPE_DATA_TYPE,
      ANY_TYPE_FORMAT_ID,
      new SingleItemTypeConverter<Object>(Object.class));

}
