/**
 * Legato is a configurable, lightweight web mapping client that can be
 * easily embedded into web pages and portals, CMS and individual web
 * applications. Legato is implemented in JavaScript and based on the
 * popular open source library OpenLayers.
 *
 * Copyright (C) 2010  disy Informationssysteme GmbH, http://www.disy.net
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package net.disy.ogc.wps.v_1_0_0.procedure;

import net.disy.ogc.wps.v_1_0_0.model.DataType;
import net.disy.ogc.wps.v_1_0_0.model.FormatId;
import net.opengis.ows.v_1_1_0.DomainMetadataType;
import net.opengis.wps.v_1_0_0.ComplexDataCombinationType;
import net.opengis.wps.v_1_0_0.ComplexDataCombinationsType;
import net.opengis.wps.v_1_0_0.ComplexDataDescriptionType;
import net.opengis.wps.v_1_0_0.SupportedComplexDataType;

public abstract class AbstractDataInputOutputBuilder {
  private final WpsProcessContext context;

  public AbstractDataInputOutputBuilder(WpsProcessContext context) {
    this.context = context;
  }

  public WpsProcessContext getContext() {
    return context;
  }

  protected void addDataCombinations(DataType<?> dataType, SupportedComplexDataType complexType) {
    ComplexDataCombinationType combinationType = context
        .getWpsObjectFactory()
        .createComplexDataCombinationType();
    complexType.setDefault(combinationType);
    ComplexDataDescriptionType format = context
        .getWpsObjectFactory()
        .createComplexDataDescriptionType();
    combinationType.setFormat(format);
    ComplexDataCombinationsType combinationsType = context
        .getWpsObjectFactory()
        .createComplexDataCombinationsType();
    complexType.setSupported(combinationsType);
    combinationsType.getFormat().add(format);
    FormatId formatId = context.getFormatRegistry().getDefaultFormat(dataType).getFormatId();
    format.setEncoding(formatId.getEncoding());
    format.setMimeType(formatId.getMimeType());
    format.setSchema(formatId.getSchemaDesignator());
  }

  protected DomainMetadataType getMetaDataType(DataType<?> dataType) {
    DomainMetadataType metadataType = context.getOwsObjectFactory().createDomainMetadataType();
    metadataType.setValue(dataType.getSchemaDesignator());
    return metadataType;
  }
}
