/**
 * Legato is a configurable, lightweight web mapping client that can be
 * easily embedded into web pages and portals, CMS and individual web
 * applications. Legato is implemented in JavaScript and based on the
 * popular open source library OpenLayers.
 *
 * Copyright (C) 2010  disy Informationssysteme GmbH, http://www.disy.net
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package net.disy.ogc.wps.v_1_0_0.procedure;

import java.text.MessageFormat;

import net.disy.ogc.wps.v_1_0_0.converter.TypeConverterRegistry;
import net.disy.ogc.wps.v_1_0_0.model.DataType;
import net.disy.ogc.wps.v_1_0_0.model.DataTypeRegistry;
import net.disy.ogc.wps.v_1_0_0.model.DefaultDataType;
import net.disy.ogc.wps.v_1_0_0.model.XmlSchemaConstants;

public class DefaultDataTypeHelper implements DataTypeHelper {
  private final DataTypeRegistry dataTypeRegistry;
  private final TypeConverterRegistry converterRegistry;

  public DefaultDataTypeHelper(
      DataTypeRegistry dataTypeRegistry,
      TypeConverterRegistry converterRegistry) {
    this.dataTypeRegistry = dataTypeRegistry;
    this.converterRegistry = converterRegistry;
  }

  @SuppressWarnings({ "unchecked", "rawtypes" })
  @Override
  public DataType<?> getDataType(ParameterDescription parameterDescription, Class<?> typeClass) {
    ParameterType parameterType = parameterDescription.getParameterType();
    final Class<?> componentType = getComponentType(typeClass);

    final DataType dataType;
    if (ParameterType.Default.equals(parameterType)) {

      net.disy.ogc.wps.v_1_0_0.annotation.DataType dt = parameterDescription.getDataType();

      DataType<?> defaultDataType = null;

      if ("##default".equals(dt.schemaDesignator())) {
        defaultDataType = dataTypeRegistry.getDefaultDataType(componentType);
      }
      else {
        try {
          defaultDataType = dataTypeRegistry.getDataType(dt.schemaDesignator());
        }
        catch (IllegalArgumentException iaex) {
          try {
            defaultDataType = dataTypeRegistry.getDefaultDataType(componentType);
          }
          catch (IllegalArgumentException iaex1) {
            defaultDataType = dataTypeRegistry
                .getDataType(XmlSchemaConstants.ANY_TYPE_SCHEMA_DESIGNATOR);
          }
        }
      }

      {
        final String schemaDesignator = "##default".equals(dt.schemaDesignator()) ? defaultDataType
            .getSchemaDesignator() : dt.schemaDesignator();
        if (schemaDesignator.equals(defaultDataType.getSchemaDesignator())) {
          dataType = defaultDataType;

        }
        else {
          dataType = new DefaultDataType(
              schemaDesignator,
              defaultDataType.getTargetClass(),
              defaultDataType.getType());
        }
      }
    }
    else {
      dataType = parameterType.getDataType(dataTypeRegistry, componentType);
    }

    if (dataType == null) {
      throw new IllegalArgumentException(MessageFormat.format(
          "Could not create the datatype for parameter description [{0}] and type class [{1}].",
          parameterDescription,
          typeClass));
    }

    if (!converterRegistry.isConversionSupported(dataType.getTargetClass(), componentType)) {
      throw new IllegalArgumentException(
          MessageFormat
              .format(
                  "Wrong type [{0}] for schema designator [{1}], could not find the appropriate converter.",
                  componentType.getName(),
                  dataType.getSchemaDesignator()));
    }
    return dataType;
  }

  private Class<?> getComponentType(Class<?> typeClass) {
    if (typeClass.isArray()) {
      return typeClass.getComponentType();
    }
    return typeClass;
  }

}
