/**
 * Legato is a configurable, lightweight web mapping client that can be
 * easily embedded into web pages and portals, CMS and individual web
 * applications. Legato is implemented in JavaScript and based on the
 * popular open source library OpenLayers.
 *
 * Copyright (C) 2010  disy Informationssysteme GmbH, http://www.disy.net
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package net.disy.ogc.wps.v_1_0_0.procedure;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;

import net.disy.ogc.wps.v_1_0_0.converter.DefaultTypeConverterRegistry;
import net.disy.ogc.wps.v_1_0_0.model.DefaultCRSRegistry;
import net.disy.ogc.wps.v_1_0_0.model.DefaultDataTypeRegistry;
import net.disy.ogc.wps.v_1_0_0.model.DefaultFormatRegistry;
import net.disy.ogc.wps.v_1_0_0.model.DefaultLiteralTypeRegistry;
import net.disy.ogc.wps.v_1_0_0.util.WpsUtils;

public class DefaultWpsProcessContext implements WpsProcessContext {

  private final JAXBContext jaxbContext;
  private final net.opengis.wps.v_1_0_0.ObjectFactory wpsObjectFactory = new net.opengis.wps.v_1_0_0.ObjectFactory();
  private final net.opengis.ows.v_1_1_0.ObjectFactory owsObjectFactory = new net.opengis.ows.v_1_1_0.ObjectFactory();
  private final DefaultDataTypeRegistry dataTypeRegistry = new DefaultDataTypeRegistry();
  private final DefaultTypeConverterRegistry converterRegistry = new DefaultTypeConverterRegistry();
  private final DefaultLiteralTypeRegistry literalTypeRegistry = new DefaultLiteralTypeRegistry();
  private final DefaultFormatRegistry formatRegistry = new DefaultFormatRegistry();
  private final DefaultCRSRegistry crsRegistry = new DefaultCRSRegistry();
  private final DefaultDataTypeHelper dataTypeHelper;

  public DefaultWpsProcessContext() {
    try {
      jaxbContext = WpsUtils.createJaxbContext();
    }
    catch (JAXBException jaxbex) {
      throw new ExceptionInInitializerError(jaxbex);
    }
    dataTypeHelper = new DefaultDataTypeHelper(getDataTypeRegistry(), getConverterRegistry());
  }
  
  @Override
  public JAXBContext getJAXBContext() {
    return jaxbContext;
  }

  @Override
  public net.opengis.wps.v_1_0_0.ObjectFactory getWpsObjectFactory() {
    return wpsObjectFactory;
  }

  @Override
  public net.opengis.ows.v_1_1_0.ObjectFactory getOwsObjectFactory() {
    return owsObjectFactory;
  }

  @Override
  public DefaultDataTypeRegistry getDataTypeRegistry() {
    return dataTypeRegistry;
  }

  @Override
  public DefaultTypeConverterRegistry getConverterRegistry() {
    return converterRegistry;
  }

  @Override
  public DefaultLiteralTypeRegistry getLiteralTypeRegistry() {
    return literalTypeRegistry;
  }

  @Override
  public DefaultFormatRegistry getFormatRegistry() {
    return formatRegistry;
  }

  @Override
  public DefaultDataTypeHelper getDataTypeHelper() {
    return dataTypeHelper;
  }

  @Override
  public DefaultCRSRegistry getCRSRegistry() {
    return this.crsRegistry;
  }

}
