/**
 * Legato is a configurable, lightweight web mapping client that can be
 * easily embedded into web pages and portals, CMS and individual web
 * applications. Legato is implemented in JavaScript and based on the
 * popular open source library OpenLayers.
 *
 * Copyright (C) 2010  disy Informationssysteme GmbH, http://www.disy.net
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package net.disy.ogc.wps.v_1_0_0.procedure;

import net.disy.ogc.gml.v_3_1_1.GmlConstants;
import net.disy.ogc.wps.v_1_0_0.model.DataType;
import net.disy.ogc.wps.v_1_0_0.model.DataTypeRegistry;
import net.disy.ogc.wps.v_1_0_0.model.XmlSchemaConstants;

public enum ParameterType {
  String() {
    @Override
    public DataType<?> getDataType(DataTypeRegistry dataTypeRegistry, Class<?> typeClass) {
      return dataTypeRegistry.getDataType(XmlSchemaConstants.STRING_SCHEMA_DESIGNATOR);
    }
  },
  Long() {
    @Override
    public DataType<?> getDataType(DataTypeRegistry dataTypeRegistry, Class<?> typeClass) {
      return dataTypeRegistry.getDataType(XmlSchemaConstants.LONG_SCHEMA_DESIGNATOR);
    }
  },
  Double() {
    @Override
    public DataType<?> getDataType(DataTypeRegistry dataTypeRegistry, Class<?> typeClass) {
      return dataTypeRegistry.getDataType(XmlSchemaConstants.DOUBLE_SCHEMA_DESIGNATOR);
    }
  },
  Boolean() {
    @Override
    public DataType<?> getDataType(DataTypeRegistry dataTypeRegistry, Class<?> typeClass) {
      return dataTypeRegistry.getDataType(XmlSchemaConstants.BOOLEAN_SCHEMA_DESIGNATOR);
    }
  },
  Date() {
    @Override
    public DataType<?> getDataType(DataTypeRegistry dataTypeRegistry, Class<?> typeClass) {
      return dataTypeRegistry.getDataType(XmlSchemaConstants.DATE_SCHEMA_DESIGNATOR);
    }
  },
  Point() {
    @Override
    public DataType<?> getDataType(DataTypeRegistry dataTypeRegistry, Class<?> typeClass) {
      return dataTypeRegistry.getDataType(GmlConstants.POINT_SCHEMA_DESIGNATOR);
    }
  },
  LineString() {
    @Override
    public DataType<?> getDataType(DataTypeRegistry dataTypeRegistry, Class<?> typeClass) {
      return dataTypeRegistry.getDataType(GmlConstants.LINE_STRING_SCHEMA_DESIGNATOR);
    }
  },
  Polygon() {
    @Override
    public DataType<?> getDataType(DataTypeRegistry dataTypeRegistry, Class<?> typeClass) {
      return dataTypeRegistry.getDataType(GmlConstants.POLYGON_SCHEMA_DESIGNATOR);
    }
  },
  Geometry() {
    @Override
    public DataType<?> getDataType(DataTypeRegistry dataTypeRegistry, Class<?> typeClass) {
      return dataTypeRegistry.getDataType(GmlConstants.GEOMETRY_SCHEMA_DESIGNATOR);
    }
  },
  Default() {
    @Override
    public DataType<?> getDataType(DataTypeRegistry dataTypeRegistry, Class<?> typeClass) {
      return dataTypeRegistry.getDefaultDataType(typeClass);
    }
  };

  public abstract DataType<?> getDataType(DataTypeRegistry dataTypeRegistry, Class<?> typeClass);
}
