/**
 * Legato is a configurable, lightweight web mapping client that can be
 * easily embedded into web pages and portals, CMS and individual web
 * applications. Legato is implemented in JavaScript and based on the
 * popular open source library OpenLayers.
 *
 * Copyright (C) 2010  disy Informationssysteme GmbH, http://www.disy.net
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package net.disy.ogc.wps.v_1_0_0.procedure;

import net.disy.ogc.ows.v_1_1_0.OwsException;
import net.disy.ogc.wps.v_1_0_0.WpsProcess;
import net.opengis.wps.v_1_0_0.DataInputsType;
import net.opengis.wps.v_1_0_0.ExecuteResponse;
import net.opengis.wps.v_1_0_0.ProcessDescriptionType;
import net.opengis.wps.v_1_0_0.ResponseFormType;

public class ProcedureWpsProcess implements WpsProcess {
  private final WpsProcessContext context;
  private final WpsProcedure procedure;
  private ProcessDescriptionType processDescription = null;

  public ProcedureWpsProcess(WpsProcessContext context, WpsProcedure procedure) {
    this.context = context;
    this.procedure = procedure;
  }

  @Override
  public ProcessDescriptionType getProcessDescription() {
    if (processDescription == null) {
      processDescription = new ProcedureWpsProcessDescriptionFactory(context)
          .createProcessDescription(procedure);
    }
    return processDescription;
  }

  @Override
  public ExecuteResponse execute(
      DataInputsType dataInputs,
      ResponseFormType responseForm,
      String language) throws OwsException {
    return new ProcedureWpsProcessExecutor(context).execute(dataInputs, responseForm, procedure);
  }
}
