/**
 * Legato is a configurable, lightweight web mapping client that can be
 * easily embedded into web pages and portals, CMS and individual web
 * applications. Legato is implemented in JavaScript and based on the
 * popular open source library OpenLayers.
 *
 * Copyright (C) 2010  disy Informationssysteme GmbH, http://www.disy.net
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package net.disy.ogc.wps.v_1_0_0.procedure;

import java.util.List;

import net.disy.ogc.wps.v_1_0_0.util.WpsUtils;
import net.opengis.ows.v_1_1_0.MetadataType;
import net.opengis.wps.v_1_0_0.ProcessDescriptionType;
import net.opengis.wps.v_1_0_0.ProcessDescriptionType.DataInputs;

public class ProcedureWpsProcessDescriptionFactory {

  private final WpsProcessContext context;

  public ProcedureWpsProcessDescriptionFactory(WpsProcessContext context) {
    this.context = context;
  }

  public ProcessDescriptionType createProcessDescription(WpsProcedure process) {
    final ProcessDescriptionType processDescription = context
        .getWpsObjectFactory()
        .createProcessDescriptionType();
    setProcessProperties(process, processDescription);
    processDescription.setDataInputs(createDataInputs(process));
    processDescription.setProcessOutputs(createProcessOutputs(process));
    return processDescription;
  }

  private void setProcessProperties(
      WpsProcedure process,
      final ProcessDescriptionType processDescription) {
    processDescription.setIdentifier(WpsUtils.createCodeType(process.getId()));
    processDescription.setTitle(WpsUtils.createLanguageStringType(process.getTitle()));
    processDescription.setAbstract(WpsUtils.createLanguageStringType(process.getAbstract()));
    final List<MetadataType> metadata = process.getMetadata();
    if (!metadata.isEmpty()) {
      processDescription.setMetadata(metadata);
    }
    processDescription.setProcessVersion("1.0.0"); //$NON-NLS-1$
    processDescription.setStatusSupported(false);
    processDescription.setStoreSupported(false);
  }

  private DataInputs createDataInputs(WpsProcedure processHelper) {
    List<InputParameterDescription> inputParameters = processHelper.getInputParameters();
    if (inputParameters.isEmpty()) {
      return null;
    }
    else {
      DataInputBuilder dataInputBuilder = new DataInputBuilder(context);
      for (InputParameterDescription parameter : inputParameters) {
        dataInputBuilder.addParameter(parameter);
      }

      return dataInputBuilder.getDataInputs();
    }
  }

  private ProcessDescriptionType.ProcessOutputs createProcessOutputs(WpsProcedure processHelper) {
    ProcessOutputDescriptionBuilder outputBuilder = new ProcessOutputDescriptionBuilder(context);

    for (OutputParameterDescription outputParameter : processHelper.getOutputParameters()) {
      outputBuilder.addOutput(outputParameter);
    }
    return outputBuilder.getProcessOutputs();
  }
}
