/**
 * Legato is a configurable, lightweight web mapping client that can be
 * easily embedded into web pages and portals, CMS and individual web
 * applications. Legato is implemented in JavaScript and based on the
 * popular open source library OpenLayers.
 *
 * Copyright (C) 2010  disy Informationssysteme GmbH, http://www.disy.net
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package net.disy.ogc.wps.v_1_0_0.procedure;

import net.disy.ogc.wps.v_1_0_0.model.DataType;
import net.disy.ogc.wps.v_1_0_0.model.DataTypeTypeVisitor;
import net.disy.ogc.wps.v_1_0_0.model.Format;
import net.disy.ogc.wps.v_1_0_0.model.FormatId;
import net.disy.ogc.wps.v_1_0_0.model.LiteralType;
import net.opengis.ows.v_1_1_0.BoundingBoxType;
import net.opengis.wps.v_1_0_0.ComplexDataType;
import net.opengis.wps.v_1_0_0.ExecuteResponse.ProcessOutputs;
import net.opengis.wps.v_1_0_0.LiteralDataType;
import net.opengis.wps.v_1_0_0.OutputDataType;
import net.opengis.wps.v_1_0_0.OutputDescriptionType;

@SuppressWarnings("nls")
public class ProcessOutputBuilder {
  private final ProcessOutputs processOutputs;
  private final WpsProcessContext context;

  public ProcessOutputBuilder(WpsProcessContext context) {
    this.context = context;
    processOutputs = context.getWpsObjectFactory().createExecuteResponseProcessOutputs();
  }

  public ProcessOutputs getProcessOutputs() {
    return processOutputs;
  }

  public <S, D> void addOutput(
      OutputDescriptionType outputDescription,
      OutputParameterDescription outputParameter,
      Class<D> outputClass,
      Object result) {
    final OutputDataType output = context.getWpsObjectFactory().createOutputDataType();
    output.setIdentifier(outputDescription.getIdentifier());
    output.setTitle(outputDescription.getTitle());
    output.setAbstract(outputDescription.getAbstract());
    processOutputs.getOutput().add(output);
    @SuppressWarnings("unchecked")
    final DataType<S> dataType = (DataType<S>) context.getDataTypeHelper().getDataType(
        outputParameter,
        outputClass);
    @SuppressWarnings("unchecked")
    final S convertedResult = context.getConverterRegistry().convertFrom(
        dataType.getTargetClass(),
        outputClass,
        (D) result);

    final net.opengis.wps.v_1_0_0.DataType outputData = context
        .getWpsObjectFactory()
        .createDataType();

    dataType.getType().accept(new DataTypeTypeVisitor() {
      @Override
      public void visitLiteral() {
        addLiteralData(outputData, dataType, convertedResult);
      }

      @Override
      public void visitComplex() {
        addComplexData(outputData, dataType, convertedResult);
      }

      @Override
      public void visitBBox() {
        outputData.setBoundingBoxData((BoundingBoxType) convertedResult);
      }
    });
    output.setData(outputData);
  }

  private <S> void addLiteralData(
      final net.opengis.wps.v_1_0_0.DataType outputData,
      DataType<S> dataType,
      S convertedResult) {
    final LiteralDataType literalOutputDataType = context
        .getWpsObjectFactory()
        .createLiteralDataType();
    outputData.setLiteralData(literalOutputDataType);

    LiteralType<S> literalType = context.getLiteralTypeRegistry().getLiteralType(dataType);
    String stringValue = literalType.toString(convertedResult);
    literalOutputDataType.setValue(stringValue);
  }

  private <S> void addComplexData(
      final net.opengis.wps.v_1_0_0.DataType outputData,
      DataType<S> dataType,
      S convertedResult) {
    ComplexDataType complexData = context.getWpsObjectFactory().createComplexDataType();
    outputData.setComplexData(complexData);
    Format<S> format = context.getFormatRegistry().getDefaultFormat(dataType);
    FormatId formatId = format.getFormatId();
    complexData.setEncoding(formatId.getEncoding());
    complexData.setMimeType(formatId.getMimeType());
    complexData.setSchema(formatId.getSchemaDesignator());
    complexData.setContent(format.marshal(convertedResult));
  }
}
