/**
 * Legato is a configurable, lightweight web mapping client that can be
 * easily embedded into web pages and portals, CMS and individual web
 * applications. Legato is implemented in JavaScript and based on the
 * popular open source library OpenLayers.
 *
 * Copyright (C) 2010  disy Informationssysteme GmbH, http://www.disy.net
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package net.disy.ogc.wps.v_1_0_0.procedure;

import java.util.List;

import net.disy.ogc.wps.v_1_0_0.model.CRS;
import net.disy.ogc.wps.v_1_0_0.model.DataType;
import net.disy.ogc.wps.v_1_0_0.model.DataTypeTypeVisitor;
import net.disy.ogc.wps.v_1_0_0.util.WpsUtils;
import net.opengis.ows.v_1_1_0.DomainMetadataType;
import net.opengis.ows.v_1_1_0.MetadataType;
import net.opengis.wps.v_1_0_0.CRSsType;
import net.opengis.wps.v_1_0_0.LiteralOutputType;
import net.opengis.wps.v_1_0_0.OutputDescriptionType;
import net.opengis.wps.v_1_0_0.ProcessDescriptionType;
import net.opengis.wps.v_1_0_0.ProcessDescriptionType.ProcessOutputs;
import net.opengis.wps.v_1_0_0.SupportedCRSsType;
import net.opengis.wps.v_1_0_0.SupportedCRSsType.Default;
import net.opengis.wps.v_1_0_0.SupportedComplexDataType;

public class ProcessOutputDescriptionBuilder extends AbstractDataInputOutputBuilder {
  private final ProcessDescriptionType.ProcessOutputs processOutputs;

  public ProcessOutputDescriptionBuilder(WpsProcessContext context) {
    super(context);
    processOutputs = getContext()
        .getWpsObjectFactory()
        .createProcessDescriptionTypeProcessOutputs();
  }

  public ProcessOutputs getProcessOutputs() {
    return processOutputs;
  }

  public void addOutput(OutputParameterDescription outputParameter) {
    processOutputs.getOutput().add(getOutputDescription(outputParameter));
  }

  private OutputDescriptionType getOutputDescription(OutputParameterDescription outputParameter) {
    final OutputDescriptionType output = getContext()
        .getWpsObjectFactory()
        .createOutputDescriptionType();
    output.setIdentifier(WpsUtils.createCodeType(outputParameter.getId()));
    output.setTitle(WpsUtils.createLanguageStringType(outputParameter.getTitle()));
    output.setAbstract(WpsUtils.createLanguageStringType(outputParameter.getAbstract()));
    final List<MetadataType> metadata = outputParameter.getMetadata();
    if (!metadata.isEmpty()) {
      output.setMetadata(metadata);
    }
    final DataType<?> dataType = getContext().getDataTypeHelper().getDataType(
        outputParameter,
        outputParameter.getParameterClass());

    dataType.getType().accept(new DataTypeTypeVisitor() {
      @Override
      public void visitLiteral() {
        addLiteralOutput(output, dataType);
      }

      @Override
      public void visitComplex() {
        addComplexOutput(output, dataType);
      }

      @Override
      public void visitBBox() {
        addBoundingBoxOutput(output, dataType);
      }
    });
    return output;
  }

  private void addLiteralOutput(final OutputDescriptionType output, DataType<?> dataType) {
    final LiteralOutputType literalOutput = getContext()
        .getWpsObjectFactory()
        .createLiteralOutputType();
    DomainMetadataType metadataType = getMetaDataType(dataType);
    literalOutput.setDataType(metadataType);
    output.setLiteralOutput(literalOutput);
  }

  private void addComplexOutput(OutputDescriptionType output, DataType<?> dataType) {
    final SupportedComplexDataType complexOutput = getContext()
        .getWpsObjectFactory()
        .createSupportedComplexDataType();
    output.setComplexOutput(complexOutput);
    addDataCombinations(dataType, complexOutput);
  }

  private void addBoundingBoxOutput(OutputDescriptionType output, DataType<?> dataType) {
    final SupportedCRSsType supportedCRSsType = getContext()
        .getWpsObjectFactory()
        .createSupportedCRSsType();

    output.setBoundingBoxOutput(supportedCRSsType);

    final Default defaultCRS = getContext().getWpsObjectFactory().createSupportedCRSsTypeDefault();
    supportedCRSsType.setDefault(defaultCRS);
    defaultCRS.setCRS(getContext().getCRSRegistry().getDefaultCRS().getName());

    final CRSsType supportedCRSs = getContext().getWpsObjectFactory().createCRSsType();

    for (final CRS supportedCRS : getContext().getCRSRegistry().getSupportedCRSs()) {
      supportedCRSs.getCRS().add(supportedCRS.getName());
    }
    supportedCRSsType.setSupported(supportedCRSs);
  }

}
