/**
 * Legato is a configurable, lightweight web mapping client that can be
 * easily embedded into web pages and portals, CMS and individual web
 * applications. Legato is implemented in JavaScript and based on the
 * popular open source library OpenLayers.
 *
 * Copyright (C) 2010  disy Informationssysteme GmbH, http://www.disy.net
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package net.disy.ogc.wps.v_1_0_0.sample;

import net.disy.ogc.gml.v_3_1_1.annotation.DefaultWKTGeometry;
import net.disy.ogc.wps.v_1_0_0.annotation.InputParameter;
import net.disy.ogc.wps.v_1_0_0.annotation.Metadata;
import net.disy.ogc.wps.v_1_0_0.annotation.OutputParameter;
import net.disy.ogc.wps.v_1_0_0.annotation.Process;
import net.disy.ogc.wps.v_1_0_0.annotation.ProcessMethod;

import com.vividsolutions.jts.geom.LineString;
import com.vividsolutions.jts.geom.Point;
import com.vividsolutions.jts.geom.Polygon;

@Process(
//
id = "hiddenInput", title = "Dummy hidden input parameters", description = "Dummy hidden input parameters abstract",
//
metadata = { @Metadata(about = "tag", href = "wps-demo:form") })
public class DummyComplexWpsProcessAnnotatedHiddenObject {

  @ProcessMethod
  @OutputParameter(
  //
  id = "hiddenInput",
  //
  title = "Dummy hidden input parameters",
  //
  description = "Just some dummy information about the input parameters",
  //
  metadata = { @Metadata(about = "visibility", href = "hidden") })
  public String dummyExecution(
      @InputParameter(id = "stringVisible", title = "Visible Input", description = "You should be able to see this (input)")//
      String stringVisible,
      @InputParameter(id = "stringHidden", title = "Hidden input", description = "You should'nt be able to see this (input)", metadata = { @Metadata(about = "visibility", href = "hidden") })//
      //@DefaultString("hiddenValue")//
      String stringHidden,
      @InputParameter(id = "pointVisible", title = "Visible Point", description = "You should be able to see this (point)")//
      Point pointVisible,
      @InputParameter(id = "PointHidden", title = "Hidden Point", description = "You should'nt be able to see this (point)", metadata = { @Metadata(about = "visibility", href = "hidden") })//
      @DefaultWKTGeometry("POINT(0 0)")//
      Point pointHidden,
      @InputParameter(id = "lineVisible", title = "Visible line test input", description = "You should be able to see this (line)")//
      LineString lineVisible,
      @InputParameter(id = "lineHidden", title = "Hidden line test input", description = "You should'nt be able to see this (line)", metadata = { @Metadata(about = "visibility", href = "hidden") })//
      @DefaultWKTGeometry("LINESTRING(0 0, 1 1, 2 2)")//
      LineString lineHidden,
      @InputParameter(id = "polyVisible", title = "Visible polygon test input", description = "You should be able to see this (polygon)")//
      Polygon polyVisible,
      @InputParameter(id = "polyHidden", title = "Hidden polygon test input", description = "You should'nt be able to see this (polygon)", metadata = { @Metadata(about = "visibility", href = "hidden") })//
      @DefaultWKTGeometry("POLYGON((0 0, 10 0, 10 10, 0 10, 0 0))")//
      Polygon polyHidden) {

    final StringBuffer sb = new StringBuffer();

    sb.append("Dummy execution result.\n"); //$NON-NLS-1$
    if (stringVisible != null) {
      sb.append("Echo [" + stringVisible + "].\n"); //$NON-NLS-1$ //$NON-NLS-2$
    }
    if (stringHidden != null) {
      sb.append("Echo [" + stringHidden + "].\n"); //$NON-NLS-1$ //$NON-NLS-2$
    }
    if (pointVisible != null) {
      sb.append("Echo [" + pointVisible + "].\n"); //$NON-NLS-1$ //$NON-NLS-2$
    }
    if (pointHidden != null) {
      sb.append("Echo [" + pointHidden + "].\n"); //$NON-NLS-1$ //$NON-NLS-2$
    }
    if (lineVisible != null) {
      sb.append("Echo [" + lineVisible + "].\n"); //$NON-NLS-1$ //$NON-NLS-2$
    }
    if (lineHidden != null) {
      sb.append("Echo [" + lineHidden + "].\n"); //$NON-NLS-1$ //$NON-NLS-2$
    }
    if (polyVisible != null) {
      sb.append("Echo [" + polyVisible + "].\n"); //$NON-NLS-1$ //$NON-NLS-2$
    }
    if (polyHidden != null) {
      sb.append("Echo [" + polyHidden + "].\n"); //$NON-NLS-1$ //$NON-NLS-2$
    }
    return sb.toString();
  }

}