/**
 * Legato is a configurable, lightweight web mapping client that can be
 * easily embedded into web pages and portals, CMS and individual web
 * applications. Legato is implemented in JavaScript and based on the
 * popular open source library OpenLayers.
 *
 * Copyright (C) 2010  disy Informationssysteme GmbH, http://www.disy.net
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package net.disy.ogc.wps.v_1_0_0.sample;

import java.io.StringWriter;

import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import net.disy.ogc.wps.v_1_0_0.annotation.DataType;
import net.disy.ogc.wps.v_1_0_0.annotation.InputParameter;
import net.disy.ogc.wps.v_1_0_0.annotation.Metadata;
import net.disy.ogc.wps.v_1_0_0.annotation.OutputParameter;
import net.disy.ogc.wps.v_1_0_0.annotation.Process;
import net.disy.ogc.wps.v_1_0_0.annotation.ProcessMethod;

import org.w3c.dom.Element;

@Process(id = "dummyLandParcelSelection", title = "Dummy Land Parcel Selection", description = "Selects a land parcel feature and returns feature XML as string", metadata = { @Metadata(about = "tag", href = "wps-demo:form") })
public class DummyLandParcelSelectionAnnotatedObject {

  @ProcessMethod
  public @OutputParameter(id = "featureAsString", title = "Feature XML as string", description = "Feature XML as string")
  String createDummyMessage(
      @InputParameter(id = "feature", title = "Land parcel", description = "Please select a land parcel", 
          dataType = @DataType(
      //
      schemaDesignator = "http://schemas.myserver.com/landparcel/1.0/schema.xsd#LandParcel")) Element feature)
      throws Exception {

    final Transformer transformer = TransformerFactory.newInstance().newTransformer();
    final StringWriter sw = new StringWriter();
    transformer.transform(new DOMSource(feature), new StreamResult(sw));
    return sw.toString();
  }

}
