/**
 * Legato is a configurable, lightweight web mapping client that can be
 * easily embedded into web pages and portals, CMS and individual web
 * applications. Legato is implemented in JavaScript and based on the
 * popular open source library OpenLayers.
 *
 * Copyright (C) 2010  disy Informationssysteme GmbH, http://www.disy.net
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package net.disy.ogc.wps.v_1_0_0.sample;

import java.util.ArrayList;
import java.util.List;

import net.disy.ogc.wps.v_1_0_0.annotation.DataType;
import net.disy.ogc.wps.v_1_0_0.annotation.InputParameter;
import net.disy.ogc.wps.v_1_0_0.annotation.OutputParameter;
import net.disy.ogc.wps.v_1_0_0.annotation.Process;
import net.disy.ogc.wps.v_1_0_0.annotation.ProcessMethod;

import com.vividsolutions.jts.geom.Coordinate;
import com.vividsolutions.jts.geom.MultiPoint;
import com.vividsolutions.jts.geom.Point;

@Process(id = "dummyRandomHeightForMultiPoint", title = "Random Height MultiPoint Service", description = "")
public class DummyRandomHeightForMultiPoint {

  @ProcessMethod
  @OutputParameter(id = "setRandomHeightForMultiPoint", title = "Set a random height for given MultiPoint", description = "Sets a random height for all points inside this MultiPoint", dataType = @DataType(schemaDesignator = "http://schemas.opengis.net/gml/3.1.1/base/gml.xsd#MultiPoint"))
  public MultiPoint setRandomHeightForMultiPoint(
      @InputParameter(id = "multipoint", title = "2D Points", description = "A selection of points where you want to define the height") MultiPoint multipoint) {
    List<Point> transformedPoints = new ArrayList<Point>();

    for (Coordinate coordinate : multipoint.getCoordinates()) {
      coordinate = new Coordinate(coordinate.x, coordinate.y, Math.random());
      transformedPoints.add(multipoint.getFactory().createPoint(coordinate));
    }

    MultiPoint transformedMultiPoint = new MultiPoint(
        transformedPoints.toArray(new Point[transformedPoints.size()]),
        multipoint.getFactory());

    return transformedMultiPoint;
  }
}
