/**
 * Legato is a configurable, lightweight web mapping client that can be
 * easily embedded into web pages and portals, CMS and individual web
 * applications. Legato is implemented in JavaScript and based on the
 * popular open source library OpenLayers.
 *
 * Copyright (C) 2010  disy Informationssysteme GmbH, http://www.disy.net
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package net.disy.ogc.wps.v_1_0_0.sample;

import java.math.BigInteger;
import java.util.ArrayList;
import java.util.GregorianCalendar;
import java.util.List;
import java.util.Locale;

import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.datatype.DatatypeFactory;

import net.disy.ogc.ows.v_1_1_0.OwsException;
import net.disy.ogc.wps.v_1_0_0.WpsProcess;
import net.disy.ogc.wps.v_1_0_0.util.WpsUtils;
import net.opengis.ows.v_1_1_0.AnyValue;
import net.opengis.ows.v_1_1_0.DomainMetadataType;
import net.opengis.ows.v_1_1_0.ExceptionReport;
import net.opengis.ows.v_1_1_0.ExceptionType;
import net.opengis.wps.v_1_0_0.DataInputsType;
import net.opengis.wps.v_1_0_0.DataType;
import net.opengis.wps.v_1_0_0.ExecuteResponse;
import net.opengis.wps.v_1_0_0.InputDescriptionType;
import net.opengis.wps.v_1_0_0.LiteralDataType;
import net.opengis.wps.v_1_0_0.LiteralInputType;
import net.opengis.wps.v_1_0_0.LiteralOutputType;
import net.opengis.wps.v_1_0_0.OutputDataType;
import net.opengis.wps.v_1_0_0.OutputDescriptionType;
import net.opengis.wps.v_1_0_0.ProcessDescriptionType;
import net.opengis.wps.v_1_0_0.ProcessFailedType;
import net.opengis.wps.v_1_0_0.ResponseFormType;
import net.opengis.wps.v_1_0_0.StatusType;
import net.opengis.wps.v_1_0_0.ExecuteResponse.ProcessOutputs;

/*
 * Do not remove - kept for demo purposes.
 */
public class DummyResponseExceptionReportWpsProcess implements WpsProcess {

  @Override
  public ExecuteResponse execute(
      DataInputsType dataInputs,
      ResponseFormType responseForm,
      String language) throws OwsException {

    final ExecuteResponse executeResponse = new ExecuteResponse();

    executeResponse.setService("WPS");
    executeResponse.setVersion("1.0.0");
    executeResponse.setLang(Locale.ENGLISH.toString());

    executeResponse.setProcess(WpsUtils.createBriefDescription(getProcessDescription()));

    final StatusType status = new StatusType();
    ProcessFailedType processFailed = new ProcessFailedType();
    processFailed.setExceptionReport(createExceptionReport());
    status.setProcessFailed(processFailed);
    try {
      status.setCreationTime(DatatypeFactory.newInstance().newXMLGregorianCalendar(
          new GregorianCalendar()));
    }
    catch (DatatypeConfigurationException ignored) {

    }
    executeResponse.setStatus(status);

    final ProcessOutputs processOutputs = new ProcessOutputs();
    final OutputDataType output = new OutputDataType();
    processOutputs.getOutput().add(output);
    output.setIdentifier(WpsUtils.createCodeType("dummy"));
    output.setTitle(WpsUtils.createLanguageStringType("Dummy output parameter"));
    DataType dataType = new DataType();
    LiteralDataType literalDataType = new LiteralDataType();
    literalDataType.setValue("Das sollte ganz und gar nicht auftauchen...");
    dataType.setLiteralData(literalDataType);
    output.setData(dataType);
    
    executeResponse.setProcessOutputs(processOutputs);

    return executeResponse;
  }

  @SuppressWarnings("serial")
  private ExceptionReport createExceptionReport() {
    ExceptionReport exceptionReport = new ExceptionReport();
    List<ExceptionType> exceptions = new ArrayList<ExceptionType>(1);
    ExceptionType exceptionType = new ExceptionType();
    exceptionType.setExceptionCode("DummyCode");
    exceptionType.setExceptionText(new ArrayList<String>() {
      {
        add("Dies ist eine Dummy-Exceptionmeldung.");
      }
    });
    exceptions.add(exceptionType);
    exceptionReport.setException(exceptions);
    return exceptionReport;
  }

  @Override
  public ProcessDescriptionType getProcessDescription() {

    final ProcessDescriptionType processDescription = new ProcessDescriptionType();

    processDescription.setAbstract(WpsUtils
        .createLanguageStringType("Dummy process that produces a response with exception report"));
    processDescription.setIdentifier(WpsUtils.createCodeType("dummyResponseWithExecutionReport"));

    processDescription.setTitle(WpsUtils
        .createLanguageStringType("Dummy process that produces a response with exception report"));

    processDescription.setProcessVersion("1.0.0");

    processDescription.setStatusSupported(false);

    createEchoInputParameter(processDescription);

    final ProcessDescriptionType.ProcessOutputs processOutputs = new ProcessDescriptionType.ProcessOutputs();

    final OutputDescriptionType output = new OutputDescriptionType();
    output.setAbstract(WpsUtils.createLanguageStringType("Dummy abstract"));
    processOutputs.getOutput().add(output);
    output.setIdentifier(WpsUtils.createCodeType("dummy"));
    output.setTitle(WpsUtils.createLanguageStringType("Dummy output parameter"));
    final LiteralOutputType literalOutput = new LiteralOutputType();
    DomainMetadataType domainMetadataType = new DomainMetadataType();
    domainMetadataType.setValue("http://www.w3.org/2001/XMLSchema.xsd#~string");
    literalOutput.setDataType(domainMetadataType);
    output.setLiteralOutput(literalOutput);

    processDescription.setProcessOutputs(processOutputs);
    return processDescription;
  }

  private void createEchoInputParameter(final ProcessDescriptionType processDescription) {
    final ProcessDescriptionType.DataInputs dataInputs = new ProcessDescriptionType.DataInputs();
    processDescription.setDataInputs(dataInputs);
    final InputDescriptionType input = new InputDescriptionType();
    dataInputs.getInput().add(input);
    input.setAbstract(WpsUtils.createLanguageStringType("Dummy abstract"));
    input.setIdentifier(WpsUtils.createCodeType("echo"));
    input.setTitle(WpsUtils.createLanguageStringType("Dummy input parameter"));
    input.setMinOccurs(BigInteger.valueOf(1));
    input.setMaxOccurs(BigInteger.valueOf(1));
    final LiteralInputType literalInput = new LiteralInputType();
    DomainMetadataType domainMetadataType = new DomainMetadataType();
    domainMetadataType.setValue("http://www.w3.org/2001/XMLSchema.xsd#~string");
    literalInput.setDataType(domainMetadataType);
    input.setLiteralData(literalInput);
    literalInput.setAnyValue(new AnyValue());
  }
}
