/**
 * Legato is a configurable, lightweight web mapping client that can be
 * easily embedded into web pages and portals, CMS and individual web
 * applications. Legato is implemented in JavaScript and based on the
 * popular open source library OpenLayers.
 *
 * Copyright (C) 2010  disy Informationssysteme GmbH, http://www.disy.net
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package net.disy.ogc.wps.v_1_0_0.sample;

import java.util.Arrays;

import net.disy.ogc.wps.v_1_0_0.annotation.OutputParameter;
import net.disy.ogc.wps.v_1_0_0.annotation.Process;
import net.disy.ogc.wps.v_1_0_0.annotation.ProcessMethod;
import net.disy.ogc.wpspd.v_1_0_0.ObjectFactory;
import net.disy.ogc.wpspd.v_1_0_0.Viewport;
import net.disy.ogc.wpspd.v_1_0_0.ViewportType;
import net.disy.ogc.wpspd.v_1_0_0.WpspdUtils;
import net.opengis.gml.v_3_1_1.EnvelopeType;

@Process(id = "dummyViewport", title = "Dummy Viewport", description = "Zooms to a bounding box")
public class DummyViewportAnnotatedObject {

  private final ObjectFactory wpspdObjectFactory = WpspdUtils.createObjectFactory();
  private final net.opengis.gml.v_3_1_1.ObjectFactory gmlObjectFactory = new net.opengis.gml.v_3_1_1.ObjectFactory();

  @ProcessMethod
  public @OutputParameter(id = "boundingBox", title = "Bounding box", description = "Targert bounding box")
  Viewport createDummyMessage() {

    final ViewportType viewport = wpspdObjectFactory.createViewportType();

    final EnvelopeType envelope = gmlObjectFactory.createEnvelopeType();

    viewport.setEnvelope(gmlObjectFactory.createEnvelope(envelope));

    envelope.setLowerCorner(gmlObjectFactory.createDirectPositionType());
    envelope.setUpperCorner(gmlObjectFactory.createDirectPositionType());
    envelope.setSrsName("urn:ogc:def:crs:EPSG::4326"); //$NON-NLS-1$

    envelope.getLowerCorner().setValue(Arrays.<Double> asList(49.007021, 8.402815));
    envelope.getUpperCorner().setValue(Arrays.<Double> asList(49.007023, 8.402813));

    return wpspdObjectFactory.createViewport(viewport);

  }

}
