/**
 * Legato is a configurable, lightweight web mapping client that can be
 * easily embedded into web pages and portals, CMS and individual web
 * applications. Legato is implemented in JavaScript and based on the
 * popular open source library OpenLayers.
 *
 * Copyright (C) 2010  disy Informationssysteme GmbH, http://www.disy.net
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package net.disy.ogc.wps.v_1_0_0.sample;

import java.math.BigInteger;
import java.util.GregorianCalendar;
import java.util.Locale;

import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.datatype.DatatypeFactory;

import net.disy.ogc.ows.v_1_1_0.OwsException;
import net.disy.ogc.wps.v_1_0_0.WpsProcess;
import net.disy.ogc.wps.v_1_0_0.util.WpsUtils;
import net.opengis.ows.v_1_1_0.AnyValue;
import net.opengis.ows.v_1_1_0.CodeType;
import net.opengis.wps.v_1_0_0.DataInputsType;
import net.opengis.wps.v_1_0_0.DataType;
import net.opengis.wps.v_1_0_0.ExecuteResponse;
import net.opengis.wps.v_1_0_0.InputDescriptionType;
import net.opengis.wps.v_1_0_0.InputType;
import net.opengis.wps.v_1_0_0.LiteralDataType;
import net.opengis.wps.v_1_0_0.LiteralInputType;
import net.opengis.wps.v_1_0_0.LiteralOutputType;
import net.opengis.wps.v_1_0_0.OutputDataType;
import net.opengis.wps.v_1_0_0.OutputDefinitionsType;
import net.opengis.wps.v_1_0_0.OutputDescriptionType;
import net.opengis.wps.v_1_0_0.ProcessDescriptionType;
import net.opengis.wps.v_1_0_0.ResponseFormType;
import net.opengis.wps.v_1_0_0.StatusType;
import net.opengis.wps.v_1_0_0.ExecuteResponse.ProcessOutputs;

/*
 * Do not remove - kept for demo purposes.
 */
public class LiteralEchoWpsProcess implements WpsProcess {

  @Override
  public ExecuteResponse execute(
      DataInputsType dataInputs,
      ResponseFormType responseForm,
      String language) throws OwsException {

    final ExecuteResponse executeResponse = new ExecuteResponse();

    executeResponse.setService("WPS");
    executeResponse.setVersion("1.0.0");
    executeResponse.setLang(Locale.ENGLISH.toString());

    //
    executeResponse.setProcess(WpsUtils.createBriefDescription(getProcessDescription()));

    final StatusType status = new StatusType();
    status.setProcessSucceeded("Ok.");
    try {
      status.setCreationTime(DatatypeFactory.newInstance().newXMLGregorianCalendar(
          new GregorianCalendar()));
    }
    catch (DatatypeConfigurationException ignored) {

    }
    executeResponse.setStatus(status);

    // If lineage is true, data inputs should be returned
    final boolean lineage = responseForm != null
        && responseForm.getResponseDocument() != null
        && responseForm.getResponseDocument().isLineage();
    if (lineage) {
      executeResponse.setDataInputs(dataInputs);
      if (!responseForm.getResponseDocument().getOutput().isEmpty()) {
        final OutputDefinitionsType outputDefinitions = new OutputDefinitionsType();
        outputDefinitions.getOutput().addAll(responseForm.getResponseDocument().getOutput());
        executeResponse.setOutputDefinitions(outputDefinitions);
      }
    }

    final ProcessOutputs processOutputs = new ProcessOutputs();
    executeResponse.setProcessOutputs(processOutputs);

    for (InputType input : dataInputs.getInput()) {
      final OutputDataType output = new OutputDataType();
      output.setIdentifier(WpsUtils.createCodeType("echo"));
      output.setTitle(WpsUtils.createLanguageStringType("Echo process"));
      final CodeType identifier = input.getIdentifier();
      if (identifier.getCodeSpace() == null && "echo".equals(identifier.getValue())) {
        final DataType data = input.getData();
        final LiteralDataType literalData = data.getLiteralData();
        final String value = literalData.getValue();

        final DataType outputData = new DataType();
        final LiteralDataType literalOutputDataType = new LiteralDataType();
        outputData.setLiteralData(literalOutputDataType);
        literalOutputDataType.setValue(value);
        output.setData(outputData);
      }
      processOutputs.getOutput().add(output);
    }

    return executeResponse;
  }

  @Override
  public ProcessDescriptionType getProcessDescription() {

    final ProcessDescriptionType processDescription = new ProcessDescriptionType();

    processDescription.setIdentifier(WpsUtils.createCodeType("echo"));

    processDescription.setTitle(WpsUtils.createLanguageStringType("Echo process"));

    processDescription.setAbstract(WpsUtils.createLanguageStringType("Echo process"));

    processDescription.setProcessVersion("1.0.0");

    processDescription.setStatusSupported(false);
//    processDescription.setStatusSupported(false);

    final ProcessDescriptionType.DataInputs dataInputs = new ProcessDescriptionType.DataInputs();
    processDescription.setDataInputs(dataInputs);

    final InputDescriptionType input = new InputDescriptionType();
    dataInputs.getInput().add(input);
    input.setIdentifier(WpsUtils.createCodeType("echo"));
    input.setAbstract(WpsUtils.createLanguageStringType("Echo input parameter"));
    input.setTitle(WpsUtils.createLanguageStringType("Echo input parameter"));
    input.setMinOccurs(BigInteger.valueOf(1));
    input.setMaxOccurs(BigInteger.valueOf(1));
    final LiteralInputType literalInput = new LiteralInputType();
    input.setLiteralData(literalInput);
    literalInput.setAnyValue(new AnyValue());

    final ProcessDescriptionType.ProcessOutputs processOutputs = new ProcessDescriptionType.ProcessOutputs();
    processDescription.setProcessOutputs(processOutputs);

    final OutputDescriptionType output = new OutputDescriptionType();
    processOutputs.getOutput().add(output);
    output.setIdentifier(WpsUtils.createCodeType("echo"));
    output.setTitle(WpsUtils.createLanguageStringType("Echo output parameter"));
    output.setAbstract(WpsUtils.createLanguageStringType("Echo output parameter"));
    final LiteralOutputType literalOutput = new LiteralOutputType();
    output.setLiteralOutput(literalOutput);

    return processDescription;
  }
}
