/**
 * Legato is a configurable, lightweight web mapping client that can be
 * easily embedded into web pages and portals, CMS and individual web
 * applications. Legato is implemented in JavaScript and based on the
 * popular open source library OpenLayers.
 *
 * Copyright (C) 2010  disy Informationssysteme GmbH, http://www.disy.net
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package net.disy.ogc.wps.v_1_0_0.sample;

import javax.xml.bind.JAXBElement;

import net.disy.ogc.wps.v_1_0_0.annotation.Metadata;
import net.disy.ogc.wps.v_1_0_0.annotation.OutputParameter;
import net.disy.ogc.wps.v_1_0_0.annotation.Process;
import net.disy.ogc.wps.v_1_0_0.annotation.ProcessMethod;
import net.disy.ogc.wps.v_1_0_0.util.WpsUtils;
import net.disy.ogc.wpspd.v_1_0_0.Geometry;
import net.disy.ogc.wpspd.v_1_0_0.GeometryType;
import net.disy.ogc.wpspd.v_1_0_0.Group;
import net.disy.ogc.wpspd.v_1_0_0.GroupType;
import net.disy.ogc.wpspd.v_1_0_0.Marker;
import net.disy.ogc.wpspd.v_1_0_0.MarkerType;
import net.disy.ogc.wpspd.v_1_0_0.Message;
import net.disy.ogc.wpspd.v_1_0_0.MessageType;
import net.disy.ogc.wpspd.v_1_0_0.WpspdUtils;
import net.opengis.gml.v_3_1_1.AbstractGeometryType;
import net.opengis.se.v_1_1_0.FillType;
import net.opengis.se.v_1_1_0.GraphicType;
import net.opengis.se.v_1_1_0.MarkType;
import net.opengis.se.v_1_1_0.PointSymbolizerType;
import net.opengis.se.v_1_1_0.StrokeType;
import net.opengis.se.v_1_1_0.SvgParameterType;
import net.opengis.se.v_1_1_0.SymbolizerType;

import org.jvnet.ogc.gml.v_3_1_1.jts.GML311ToJTSConstants;
import org.jvnet.ogc.gml.v_3_1_1.jts.JTSToGML311GeometryConverter;

import com.vividsolutions.jts.geom.Coordinate;
import com.vividsolutions.jts.geom.GeometryFactory;
import com.vividsolutions.jts.geom.Point;

@Process(id = "markersDemo", title = "Markers demo", description = "Demonstrates various markers", metadata = { @Metadata(about = "tag", href = "wps-demo:form") })
public class MarkersDemoAnnotatedObject {

  private final GeometryFactory geometryFactory = GML311ToJTSConstants.DEFAULT_GEOMETRY_FACTORY;
  private final JTSToGML311GeometryConverter geometryConverter = new JTSToGML311GeometryConverter();
  private final net.disy.ogc.wpspd.v_1_0_0.ObjectFactory wpspdObjectFactory = WpspdUtils.createObjectFactory();
  private final net.opengis.se.v_1_1_0.ObjectFactory seObjectFactory = new net.opengis.se.v_1_1_0.ObjectFactory();
  private final net.opengis.gml.v_3_1_1.ObjectFactory gmlObjectFactory = new net.opengis.gml.v_3_1_1.ObjectFactory();

  @ProcessMethod
  public @OutputParameter(id = "markers", title = "Markers", description = "Group of markers")
  Group createMarkers() {

    final GroupType groupType = wpspdObjectFactory.createGroupType();
    final Group group = wpspdObjectFactory.createGroup(groupType);

    {
      final MessageType message = wpspdObjectFactory.createMessageType();

      message.setPresentationDirectiveTitle(WpsUtils.createLanguageStringType("You've got mail"));
      message.setAbstract(WpsUtils.createLanguageStringType("Message description"));
      message.setContent("How do you do?");
      message.setType("info");

      groupType.getPresentationDirective().add(wpspdObjectFactory.createMessage(message));
    }

    {
      // 49, 8

      final MarkerType markerType = wpspdObjectFactory.createMarkerType();
      final Marker marker = wpspdObjectFactory.createMarker(markerType);

      groupType.getPresentationDirective().add(marker);

      final Point point = geometryFactory.createPoint(new Coordinate(49, 8));

      @SuppressWarnings("unchecked")
      final JAXBElement<AbstractGeometryType> pointGeometry = (JAXBElement<AbstractGeometryType>) geometryConverter
          .createElement(point);

      final PointSymbolizerType pointSymbolizerType = seObjectFactory.createPointSymbolizerType();
      final JAXBElement<PointSymbolizerType> pointSymbolizer =

      seObjectFactory.createPointSymbolizer(pointSymbolizerType);
      // Unbelievable
      final JAXBElement<SymbolizerType> symbolizer = (JAXBElement<SymbolizerType>) (Object) pointSymbolizer;
      final GraphicType graphicType = seObjectFactory.createGraphicType();
      pointSymbolizerType.setGraphic(graphicType);
      final MarkType markType = seObjectFactory.createMarkType();
      graphicType.getExternalGraphicOrMark().add(markType);
      final FillType fillType = seObjectFactory.createFillType();
      markType.setFill(fillType);
      final StrokeType strokeType = seObjectFactory.createStrokeType();
      markType.setStroke(strokeType);
      {
        final SvgParameterType svgParameterType = seObjectFactory.createSvgParameterType();
        svgParameterType.setName("fill");
        svgParameterType.getContent().add("#ff0000");
        fillType.getSvgParameter().add(svgParameterType);
      }
      {
        final SvgParameterType svgParameterType = seObjectFactory.createSvgParameterType();
        svgParameterType.setName("stroke");
        svgParameterType.getContent().add("#0000FF");
        strokeType.getSvgParameter().add(svgParameterType);
      }
      {
        final SvgParameterType svgParameterType = seObjectFactory.createSvgParameterType();
        svgParameterType.setName("stroke-width");
        svgParameterType.getContent().add("2");
        strokeType.getSvgParameter().add(svgParameterType);
      }

      final GeometryType geometryType = wpspdObjectFactory.createGeometryType();

      final Geometry geometry = wpspdObjectFactory.createGeometry(geometryType);

      geometryType.setGeometry((JAXBElement<AbstractGeometryType>) pointGeometry);
      geometryType.setSymbolizer(symbolizer);
      markerType.setGeometry(geometry);

      final MessageType messageType = wpspdObjectFactory.createMessageType();
      final Message message = wpspdObjectFactory.createMessage(messageType);
      messageType.setContent("Text");
      markerType.setMessage(message);

    }

    {
      // LineString
      // 49.5 8 50 8 50 8.5
    }

    {
      // Polygon
      // 50.5 8| 51 8| 51 8.5| 50.5 8.5| 50.5 8 
    }

    return group;
  }

}
