/**
 * Legato is a configurable, lightweight web mapping client that can be
 * easily embedded into web pages and portals, CMS and individual web
 * applications. Legato is implemented in JavaScript and based on the
 * popular open source library OpenLayers.
 *
 * Copyright (C) 2010  disy Informationssysteme GmbH, http://www.disy.net
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package net.disy.ogc.wps.v_1_0_0.sample;

import net.disy.ogc.wps.v_1_0_0.annotation.InputParameter;
import net.disy.ogc.wps.v_1_0_0.annotation.Metadata;
import net.disy.ogc.wps.v_1_0_0.annotation.OutputParameter;
import net.disy.ogc.wps.v_1_0_0.annotation.Process;
import net.disy.ogc.wps.v_1_0_0.annotation.ProcessMethod;

import com.vividsolutions.jts.geom.Coordinate;
import com.vividsolutions.jts.geom.CoordinateSequence;
import com.vividsolutions.jts.geom.Geometry;
import com.vividsolutions.jts.geom.LineString;
import com.vividsolutions.jts.geom.Point;
import com.vividsolutions.jts.geom.impl.CoordinateArraySequence;

@Process(
//
id = "bboxPoint",
//
title = "Point bounding box calculation process",
//
description = "Description of the point bounding box calculation process",
//
metadata = {
    @Metadata(about = "shortcut icon", href = "http://www.disy.net/fileadmin/template/html/favicon.ico"),
    @Metadata(about = "tag", href = "wps-demo:form"),
    @Metadata(about = "tag", href = "wps-demo:toolbar") })
public class PointBoundingBoxAnnotatedObject {
  @ProcessMethod
  @OutputParameter(
  //
  id = "bboxPoint",
  //
  title = "Point to calculate BBox",
  //
  description = "Point to calculate BBox")
  public Geometry bboxPolygon(@InputParameter(
  //
      id = "echo",
      //
      title = "input point in JS form",
      //
      description = "Description of the input point in JS form") Point input) {
    //create a 1000x1000 bounding box around the point
    Coordinate[] coordinates = new Coordinate[] {
        new Coordinate(input.getX() - 500, input.getY() - 500, 0),
        new Coordinate(input.getX() + 500, input.getY() + 500, 0)
    };
    CoordinateSequence coordinateSequence = new CoordinateArraySequence(coordinates);
    LineString boxLineString = new LineString(coordinateSequence, input.getFactory());
    return boxLineString.getEnvelope();
  }

}
