/**
 * Legato is a configurable, lightweight web mapping client that can be
 * easily embedded into web pages and portals, CMS and individual web
 * applications. Legato is implemented in JavaScript and based on the
 * popular open source library OpenLayers.
 *
 * Copyright (C) 2010  disy Informationssysteme GmbH, http://www.disy.net
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package net.disy.ogc.wps.v_1_0_0.sample;

import java.net.URL;
import java.text.MessageFormat;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;

import net.disy.ogc.wps.v_1_0_0.util.Validate;

public class SampleAnnotatedObject<T> implements ParameterlessAnnotatedObject<T> {

  private final Class<T> outputParameterClass;

  private final T result;

  public SampleAnnotatedObject(
      JAXBContext context,
      Class<T> outputParameterClass,
      String resourceName) {
    Validate.notNull(context);
    Validate.notNull(outputParameterClass);
    Validate.notNull(resourceName);
    this.outputParameterClass = outputParameterClass;
    this.result = createResult(context, outputParameterClass, resourceName);
  }

  public Class<T> getOutputParameterClass() {
    return outputParameterClass;
  }

  @SuppressWarnings("unchecked")
  private T createResult(JAXBContext context, Class<T> outputParameterClass, String resourceName) {

    final URL resourceURL = getClass().getResource(resourceName);
    if (resourceURL == null) {
      throw new IllegalArgumentException(MessageFormat.format(
          "Resource [{0}] could not be found.",
          resourceName));
    }
    try {
      Object object = context.createUnmarshaller().unmarshal(resourceURL);
      if (outputParameterClass.isAssignableFrom(object.getClass())) {
        return (T) object;
      }
      else {
        throw new IllegalArgumentException(MessageFormat.format(
            "Unmarshalled object class [{0}] does not match the expected class [{1}].",
            object.getClass(),
            outputParameterClass));
      }
    }
    catch (JAXBException jaxbex) {
      throw new IllegalArgumentException(MessageFormat.format(
          "Resource [{0}] could not be parsed.",
          resourceName), jaxbex);
    }

  }

  @Override
  public T execute() {
    return result;
  }

}
