/**
 * Legato is a configurable, lightweight web mapping client that can be
 * easily embedded into web pages and portals, CMS and individual web
 * applications. Legato is implemented in JavaScript and based on the
 * popular open source library OpenLayers.
 *
 * Copyright (C) 2010  disy Informationssysteme GmbH, http://www.disy.net
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package net.disy.ogc.wps.v_1_0_0.servlet;

import java.io.IOException;
import java.util.Map;
import java.util.Map.Entry;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBElement;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import javax.xml.namespace.QName;
import javax.xml.transform.Source;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;

import net.disy.commons.collections15.util.MultiMapUtils;
import net.disy.ogc.ows.v_1_1_0.InvalidParameterValueException;
import net.disy.ogc.ows.v_1_1_0.OwsException;
import net.disy.ogc.wps.v_1_0_0.WpsConstants;
import net.disy.ogc.wps.v_1_0_0.WpsOperations;
import net.disy.ogc.wps.v_1_0_0.omm.DescribeProcessUnmapper;
import net.disy.ogc.wps.v_1_0_0.omm.GetCapabilitiesUnmapper;
import net.opengis.wps.v_1_0_0.DescribeProcess;
import net.opengis.wps.v_1_0_0.Execute;
import net.opengis.wps.v_1_0_0.ExecuteResponse;
import net.opengis.wps.v_1_0_0.GetCapabilities;
import net.opengis.wps.v_1_0_0.ProcessDescriptions;
import net.opengis.wps.v_1_0_0.WPSCapabilitiesType;

import org.apache.commons.collections15.MultiMap;
import org.apache.commons.collections15.multimap.MultiHashMap;

public abstract class AbstractWpsOperationsServlet extends HttpServlet {

  private static final long serialVersionUID = 1L;

  public abstract WpsOperations getWpsOperations(HttpServletRequest request)
      throws IllegalStateException;

  public abstract JAXBContext getContext() throws IllegalStateException;

  public AbstractWpsOperationsServlet() {
    super();
  }

  @Override
  protected void doGet(HttpServletRequest request, HttpServletResponse response)
      throws ServletException,
      IOException {

    final MultiMap<String, String> source = createRequestMap(request);

    try {
      try {
        String requestParameterValue = MultiMapUtils.get(source, "request"); //$NON-NLS-1$
        if ("GetCapabilities".equals(requestParameterValue)) { //$NON-NLS-1$

          final GetCapabilitiesUnmapper unmapper = new GetCapabilitiesUnmapper();
          final GetCapabilities input = unmapper.unmap(source);
          final WPSCapabilitiesType capabilities = getWpsOperations(request).getCapabilities(input);

          marshal(new JAXBElement<WPSCapabilitiesType>(new QName(
              "http://www.opengis.net/wps/1.0.0", //$NON-NLS-1$
              "Capabilities"), WPSCapabilitiesType.class, capabilities), //$NON-NLS-1$

              response);

        }
        else if ("DescribeProcess".equals(requestParameterValue)) { //$NON-NLS-1$

          final DescribeProcessUnmapper unmapper = new DescribeProcessUnmapper();
          final DescribeProcess input = unmapper.unmap(source);
          final ProcessDescriptions processDescriptions = getWpsOperations(request)
              .describeProcess(input);

          marshal(processDescriptions, response);

        }
        else {
          throw new InvalidParameterValueException("request", requestParameterValue); //$NON-NLS-1$
        }
      }
      catch (OwsException owsex) {
        handleOwsException(response, owsex);

      }
    }
    catch (JAXBException jaxbex) {
      throw new ServletException(jaxbex);
    }
  }

  private void marshal(final Object object, HttpServletResponse response)
      throws JAXBException,
      IOException {
    Marshaller marshaller = getContext().createMarshaller();

    marshaller.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, Boolean.TRUE);
    marshaller.setProperty(
        "com.sun.xml.bind.namespacePrefixMapper",
        WpsConstants.NAMESPACE_PREFIX_MAPPER);
    marshaller.marshal(object, new StreamResult(response.getOutputStream()));

  }

  private void handleOwsException(HttpServletResponse response, OwsException owsex)
      throws JAXBException,
      IOException {
    response.setStatus(HttpServletResponse.SC_INTERNAL_SERVER_ERROR);
    getContext().createMarshaller().marshal(
        owsex.getExceptionReport(),
        new StreamResult(response.getOutputStream()));
  }

  private MultiMap<String, String> createRequestMap(HttpServletRequest request) {
    @SuppressWarnings("unchecked")
    final Map<String, String[]> draftMap = request.getParameterMap();
    final MultiMap<String, String> source = new MultiHashMap<String, String>(draftMap.size());
    for (Entry<String, String[]> entry : draftMap.entrySet()) {
      for (String value : entry.getValue()) {
        final String key = entry.getKey();
        source.put(key.toLowerCase(), value);
      }
    }
    return source;
  }

  @Override
  protected void doPost(HttpServletRequest request, HttpServletResponse response)
      throws ServletException,
      IOException {
    final MultiMap<String, String> requestMap = createRequestMap(request);

    try {
      try {
        String requestParameterValue = MultiMapUtils.get(requestMap, "request"); //$NON-NLS-1$
        if ("Execute".equals(requestParameterValue)) { //$NON-NLS-1$

          final Source source = new StreamSource(request.getReader());
          final Execute execute = getContext()
              .createUnmarshaller()
              .unmarshal(source, Execute.class)
              .getValue();
          final ExecuteResponse executeResponse = getWpsOperations(request).execute(execute);
          marshal(executeResponse, response);
        }
      }
      catch (OwsException owsex) {
        handleOwsException(response, owsex);
      }
    }
    catch (JAXBException jaxbex) {
      throw new ServletException(jaxbex);
    }

  }

}