/**
 * Legato is a configurable, lightweight web mapping client that can be
 * easily embedded into web pages and portals, CMS and individual web
 * applications. Legato is implemented in JavaScript and based on the
 * popular open source library OpenLayers.
 *
 * Copyright (C) 2010  disy Informationssysteme GmbH, http://www.disy.net
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package net.disy.ogc.wps.v_1_0_0.servlet;

import java.util.Arrays;

import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;

import net.disy.ogc.wps.v_1_0_0.WpsProcessRegistry;
import net.disy.ogc.wps.v_1_0_0.annotation.ProcessForClassNameAdder;
import net.disy.ogc.wps.v_1_0_0.procedure.WpsProcessContext;

import org.apache.commons.lang.StringUtils;

public class WpsConfigurationServlet extends HttpServlet {

  private static final long serialVersionUID = 1L;
  public static final String PROCESSES_INIT_PARAMETER_NAME = "processes"; //$NON-NLS-1$

  public WpsProcessRegistry getWpsProcessRegistry() throws IllegalStateException {
    final Object wpsProcessRegistry = getServletContext().getAttribute(
        WpsBaseInitializationServlet.WPS_PROCESS_REGISTRY_BEAN_NAME);
    if (wpsProcessRegistry == null) {
      throw new IllegalStateException("Bean [" //$NON-NLS-1$
          + WpsBaseInitializationServlet.WPS_PROCESS_REGISTRY_BEAN_NAME
          + "] could not be found in the servlet context."); //$NON-NLS-1$
    }
    return (WpsProcessRegistry) wpsProcessRegistry;
  }

  public WpsProcessContext getWpsProcessContext() throws IllegalStateException {
    final Object wpsProcessContext = getServletContext().getAttribute(
        WpsBaseInitializationServlet.WPS_PROCESS_CONTEXT_BEAN_NAME);
    if (wpsProcessContext == null) {
      throw new IllegalStateException("Bean [" //$NON-NLS-1$
          + WpsBaseInitializationServlet.WPS_PROCESS_CONTEXT_BEAN_NAME
          + "] could not be found in the servlet context."); //$NON-NLS-1$
    }
    return (WpsProcessContext) wpsProcessContext;
  }

  @Override
  public void init(ServletConfig config) throws ServletException {
    super.init(config);
    String processes = config.getInitParameter(PROCESSES_INIT_PARAMETER_NAME);
    if (processes != null) {
      final String[] processClassNames = StringUtils.split(processes, ',');
      try {
        new ProcessForClassNameAdder(getWpsProcessContext(), getWpsProcessRegistry())
            .addProcessesForClassNames(Arrays.asList(processClassNames));
      }
      catch (IllegalArgumentException e) {
        throw new ServletException(e);
      }
    }
  }

}
