/**
 * Legato is a configurable, lightweight web mapping client that can be
 * easily embedded into web pages and portals, CMS and individual web
 * applications. Legato is implemented in JavaScript and based on the
 * popular open source library OpenLayers.
 *
 * Copyright (C) 2010  disy Informationssysteme GmbH, http://www.disy.net
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package net.disy.ogc.wps.v_1_0_0.servlet;

import java.net.MalformedURLException;
import java.net.URL;
import java.util.Locale;

import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.xml.bind.JAXBException;
import javax.xml.transform.stream.StreamSource;

import net.disy.ogc.wps.v_1_0_0.DefaultWpsOperationsFactory;
import net.disy.ogc.wps.v_1_0_0.util.WpsOperationsUtilities;
import net.opengis.ows.v_1_1_0.ServiceIdentification;
import net.opengis.ows.v_1_1_0.ServiceProvider;

import org.apache.commons.lang.LocaleUtils;

public class WpsOperationsInitializationServlet extends HttpServlet {
  private static final long serialVersionUID = 1L;

  public static final String WPS_OPERATIONS_FACTORY_NAME = DefaultWpsOperationsFactory.class
      .getName();
  public static final String SERVLET_NAME_INIT_PARAMETER_NAME = "servletName"; //$NON-NLS-1$
  public static final String SERVICE_IDENTIFICATION_INIT_PARAMETER_NAME = "serviceIdentification"; //$NON-NLS-1$
  public static final String SERVICE_PROVIDER_INIT_PARAMETER_NAME = "serviceProvider"; //$NON-NLS-1$
  public static final String DEFAULT_LANGUAGE_INIT_PARAMETER_NAME = "defaultLanguage"; //$NON-NLS-1$

  @Override
  public void init(ServletConfig config) throws ServletException {
    ServiceIdentification serviceIdentification = null;
    final String serviceIdentificationResource = config
        .getInitParameter(SERVICE_IDENTIFICATION_INIT_PARAMETER_NAME);

    if (serviceIdentificationResource != null) {
      try {
        final URL url = config.getServletContext().getResource(serviceIdentificationResource);
        if (url == null) {
          throw new ServletException("Could not find service identificaiton resource [" //$NON-NLS-1$
              + serviceIdentificationResource
              + "]."); //$NON-NLS-1$
        }
        final ServiceIdentification value = WpsOperationsUtilities
            .getWpsProcessContext(config.getServletContext())
            .getJAXBContext()
            .createUnmarshaller()
            .unmarshal(new StreamSource(url.toString()), ServiceIdentification.class)
            .getValue();
        serviceIdentification = value;
      }
      catch (MalformedURLException muex) {
        throw new ServletException("Could not load service identificaiton resource [" //$NON-NLS-1$
            + serviceIdentificationResource
            + "].", muex); //$NON-NLS-1$
      }
      catch (JAXBException jaxbex) {
        throw new ServletException("Could not load service identification resource [" //$NON-NLS-1$
            + serviceIdentificationResource
            + "].", jaxbex); //$NON-NLS-1$
      }
    }

    final String serviceProviderResource = config
        .getInitParameter(SERVICE_PROVIDER_INIT_PARAMETER_NAME);

    ServiceProvider serviceProvider = null;
    if (serviceProviderResource != null) {
      try {
        final URL url = config.getServletContext().getResource(serviceProviderResource);
        if (url == null) {
          throw new ServletException("Could not find service provider resource [" //$NON-NLS-1$
              + serviceProviderResource
              + "]."); //$NON-NLS-1$
        }
        final ServiceProvider value = WpsOperationsUtilities
            .getWpsProcessContext(config.getServletContext())
            .getJAXBContext()
            .createUnmarshaller()
            .unmarshal(new StreamSource(url.toString()), ServiceProvider.class)
            .getValue();
        serviceProvider = value;
      }
      catch (MalformedURLException muex) {
        throw new ServletException("Could not load service provider resource [" //$NON-NLS-1$
            + serviceProviderResource
            + "].", muex); //$NON-NLS-1$
      }
      catch (JAXBException jaxbex) {
        throw new ServletException("Could not load service provider resource [" //$NON-NLS-1$
            + serviceProviderResource
            + "].", jaxbex); //$NON-NLS-1$
      }
    }

    final String defaultLanguage = config.getInitParameter(DEFAULT_LANGUAGE_INIT_PARAMETER_NAME);

    Locale defaultLocale = null;
    if (defaultLanguage != null) {
      defaultLocale = LocaleUtils.toLocale(defaultLanguage);
    }

    DefaultWpsOperationsFactory wpsOperationsFactory = new DefaultWpsOperationsFactory(
        serviceIdentification,
        serviceProvider,
        defaultLocale,
        WpsOperationsUtilities.getWpsProcessRegistry(config.getServletContext()),
        config.getInitParameter(SERVLET_NAME_INIT_PARAMETER_NAME));

    config.getServletContext().setAttribute(WPS_OPERATIONS_FACTORY_NAME, wpsOperationsFactory);
  }

}
