/**
 * Legato is a configurable, lightweight web mapping client that can be
 * easily embedded into web pages and portals, CMS and individual web
 * applications. Legato is implemented in JavaScript and based on the
 * popular open source library OpenLayers.
 *
 * Copyright (C) 2010  disy Informationssysteme GmbH, http://www.disy.net
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package net.disy.ogc.wps.v_1_0_0.util;

import javax.servlet.ServletContext;
import javax.servlet.http.HttpServletRequest;

import net.disy.commons.servlet.util.ServletUrlProvider;
import net.disy.ogc.wps.v_1_0_0.DefaultWpsOperationsFactory;
import net.disy.ogc.wps.v_1_0_0.LoggingWpsOperationdDecorator;
import net.disy.ogc.wps.v_1_0_0.WpsOperations;
import net.disy.ogc.wps.v_1_0_0.WpsProcessRegistry;
import net.disy.ogc.wps.v_1_0_0.procedure.WpsProcessContext;
import net.disy.ogc.wps.v_1_0_0.servlet.WpsBaseInitializationServlet;
import net.disy.ogc.wps.v_1_0_0.servlet.WpsOperationsInitializationServlet;

public class WpsOperationsUtilities {
  public static WpsOperations createWpsOperations(ServletContext context, HttpServletRequest request) {
    DefaultWpsOperationsFactory wpsOperationsFactory = (DefaultWpsOperationsFactory) context
        .getAttribute(WpsOperationsInitializationServlet.WPS_OPERATIONS_FACTORY_NAME);
    final WpsOperations wpsOperations = wpsOperationsFactory
        .createWpsOperations(new ServletUrlProvider(request));

    return new LoggingWpsOperationdDecorator(wpsOperations, getWpsProcessContext(context)
        .getJAXBContext());
  }

  public static WpsProcessContext getWpsProcessContext(ServletContext servletContext)
      throws IllegalStateException {
    final Object context = servletContext
        .getAttribute(WpsBaseInitializationServlet.WPS_PROCESS_CONTEXT_BEAN_NAME);
    if (context == null) {
      throw new IllegalStateException("The bean [" //$NON-NLS-1$
          + WpsBaseInitializationServlet.WPS_PROCESS_CONTEXT_BEAN_NAME
          + "] could not be found in the servlet context."); //$NON-NLS-1$
    }
    return (WpsProcessContext) context;
  }

  public static WpsProcessRegistry getWpsProcessRegistry(ServletContext servletContext)
      throws IllegalStateException {
    final Object wpsProcessRegistry = servletContext
        .getAttribute(WpsBaseInitializationServlet.WPS_PROCESS_REGISTRY_BEAN_NAME);
    if (wpsProcessRegistry == null) {
      throw new IllegalStateException("The bean [" //$NON-NLS-1$
          + WpsBaseInitializationServlet.WPS_PROCESS_REGISTRY_BEAN_NAME
          + "] could not be found in the servlet context."); //$NON-NLS-1$
    }
    return (WpsProcessRegistry) wpsProcessRegistry;
  }
}
