/**
 * Legato is a configurable, lightweight web mapping client that can be
 * easily embedded into web pages and portals, CMS and individual web
 * applications. Legato is implemented in JavaScript and based on the
 * popular open source library OpenLayers.
 *
 * Copyright (C) 2010  disy Informationssysteme GmbH, http://www.disy.net
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package net.disy.ogc.wps.v_1_0_0.util;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.List;

import net.disy.ogc.wps.v_1_0_0.annotation.Metadata;
import net.opengis.ows.v_1_1_0.MetadataType;

import org.apache.commons.collections15.CollectionUtils;
import org.apache.commons.collections15.Predicate;
import org.apache.commons.lang.StringUtils;

public class WpsProcessUtilities {
  public static <E, T extends Throwable> E getSingleRequired(
      E[] array,
      Predicate<? super E> predicate,
      T throwable) throws T {
    return getSingleRequired(Arrays.asList(array), predicate, throwable);
  }

  public static <E, T extends Throwable> E getSingleRequired(
      List<E> list,
      Predicate<? super E> predicate,
      T throwable) throws T {
    Collection<E> found = CollectionUtils.select(list, predicate);
    if (found.size() != 1) {
      throw throwable;
    }
    return found.iterator().next();
  }

  public static List<MetadataType> getMetadata(Metadata[] mds) {
    if (mds == null || mds.length == 0) {
      return Collections.emptyList();
    }
    else {
      final List<MetadataType> metadata = new ArrayList<MetadataType>(mds.length);
      for (Metadata md : mds) {
        final MetadataType item = new MetadataType();
        final String about = md.about();
        final String href = md.href();
        final String title = md.title();
        if (!StringUtils.isEmpty(about)) {
          item.setAbout(about);
        }
        if (!StringUtils.isEmpty(href)) {
          item.setHref(href);
        }
        if (!StringUtils.isEmpty(title)) {
          item.setTitle(title);
        }
        metadata.add(item);
      }

      return metadata;
    }
  }

}
