/**
 * Legato is a configurable, lightweight web mapping client that can be
 * easily embedded into web pages and portals, CMS and individual web
 * applications. Legato is implemented in JavaScript and based on the
 * popular open source library OpenLayers.
 *
 * Copyright (C) 2010  disy Informationssysteme GmbH, http://www.disy.net
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package net.disy.ogc.wps.v_1_0_0.util;

import java.net.URL;
import java.util.HashMap;
import java.util.Map;

import javax.xml.XMLConstants;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.transform.stream.StreamSource;
import javax.xml.validation.Schema;
import javax.xml.validation.SchemaFactory;

import net.disy.ogc.gml.v_3_1_1.GmlConstants;
import net.disy.ogc.wps.v_1_0_0.WpsConstants;
import net.disy.ogc.wpspd.v_1_0_0.WpspdConstants;
import net.opengis.ows.v_1_1_0.CodeType;
import net.opengis.ows.v_1_1_0.LanguageStringType;
import net.opengis.wps.v_1_0_0.ProcessBriefType;
import net.opengis.wps.v_1_0_0.ProcessDescriptionType;

import org.xml.sax.SAXException;

public class WpsUtils {

  private WpsUtils() {
    //should never happen
  }

  public static ProcessBriefType createBriefDescription(ProcessDescriptionType processDescription) {

    final ProcessBriefType briefProcessDescription = new ProcessBriefType();

    if (processDescription.getIdentifier() != null) {
      briefProcessDescription.setIdentifier(processDescription.getIdentifier());
    }

    if (processDescription.getTitle() != null) {
      briefProcessDescription.setTitle(processDescription.getTitle());
    }

    if (processDescription.getAbstract() != null) {
      briefProcessDescription.setAbstract(processDescription.getAbstract());
    }

    if (processDescription.getMetadata().isEmpty()) {
      briefProcessDescription.getMetadata().addAll(processDescription.getMetadata());
    }

    if (!processDescription.getProfile().isEmpty()) {
      briefProcessDescription.getProfile().addAll(processDescription.getProfile());
    }

    if (processDescription.getWSDL() != null) {
      briefProcessDescription.setWSDL(processDescription.getWSDL());
    }

    if (processDescription.getProcessVersion() != null) {
      briefProcessDescription.setProcessVersion(processDescription.getProcessVersion());
    }
    return briefProcessDescription;
  }

  public static CodeType createCodeType(String value) {
    final CodeType code = new CodeType();
    code.setValue(value);
    return code;
  }

  public static LanguageStringType createLanguageStringType(String value) {
    final LanguageStringType title = new LanguageStringType();
    title.setValue(value);
    return title;
  }

  public static Schema createSchema() throws SAXException {

    final SchemaFactory schemaFactory = SchemaFactory
        .newInstance(XMLConstants.W3C_XML_SCHEMA_NS_URI);

    final URL schemaResource = WpsUtils.class.getClassLoader().getResource("wps/1.0.0/wpsAll.xsd"); //$NON-NLS-1$

    final Schema schema = schemaFactory.newSchema(new StreamSource(schemaResource.toString()));
    return schema;

  }

  public static JAXBContext createJaxbContext() throws JAXBException {
    final JAXBContext context = JAXBContext.newInstance(WpsConstants.CONTEXT_PATH + ":" //$NON-NLS-1$
        + GmlConstants.CONTEXT_PATH
        + ":" //$NON-NLS-1$
        + WpspdConstants.CONTEXT_PATH);

    return context;
  }

}
