/**
 * Legato is a configurable, lightweight web mapping client that can be
 * easily embedded into web pages and portals, CMS and individual web
 * applications. Legato is implemented in JavaScript and based on the
 * popular open source library OpenLayers.
 *
 * Copyright (C) 2010  disy Informationssysteme GmbH, http://www.disy.net
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package net.disy.ogc.wpspd.v_1_0_0;

import javax.xml.namespace.QName;

import net.disy.ogc.wps.v_1_0_0.model.DataType;
import net.disy.ogc.wps.v_1_0_0.model.SchemaReference;

public class WpspdSchemaConstants {

  private WpspdSchemaConstants() {

  }

  public static String LOCATION = "http://schemas.disy.net/wps-pd/1.0.0/wps-pd.xsd"; //$NON-NLS-1$
  public static String NAMESPACE_URI = "http://www.disy.net/pd"; //$NON-NLS-1$

  public static SchemaReference WPS_PD = new SchemaReference(LOCATION, NAMESPACE_URI, "wps-pd"); //$NON-NLS-1$

  public final static String LINK_LOCAL_NAME = "Link"; //$NON-NLS-1$
  public final static String MESSAGE_LOCAL_NAME = "Message"; //$NON-NLS-1$
  public final static String VIEWPORT_LOCAL_NAME = "Viewport"; //$NON-NLS-1$
  public final static String MARKER_LOCAL_NAME = "Marker"; //$NON-NLS-1$
  public final static String GEOMETRY_LOCAL_NAME = "Geometry"; //$NON-NLS-1$
  public final static String GROUP_LOCAL_NAME = "Group"; //$NON-NLS-1$

  public static String LINK_SCHEMA_DESIGNATOR = WPS_PD.elementDesignator(LINK_LOCAL_NAME);
  public static String MESSAGE_SCHEMA_DESIGNATOR = WPS_PD.elementDesignator(MESSAGE_LOCAL_NAME);
  public static String VIEWPORT_SCHEMA_DESIGNATOR = WPS_PD.elementDesignator(VIEWPORT_LOCAL_NAME);
  public static String MARKER_SCHEMA_DESIGNATOR = WPS_PD.elementDesignator(MARKER_LOCAL_NAME);
  public static String GEOMETRY_SCHEMA_DESIGNATOR = WPS_PD.elementDesignator(GEOMETRY_LOCAL_NAME);
  public static String GROUP_SCHEMA_DESIGNATOR = WPS_PD.elementDesignator(GROUP_LOCAL_NAME);

  public static QName LINK_ELEMENT_NAME = WPS_PD.qname(LINK_LOCAL_NAME);
  public static QName MESSAGE_ELEMENT_NAME = WPS_PD.qname(MESSAGE_LOCAL_NAME);
  public static QName VIEWPORT_ELEMENT_NAME = WPS_PD.qname(VIEWPORT_LOCAL_NAME);
  public static QName MARKER_ELEMENT_NAME = WPS_PD.qname(MARKER_LOCAL_NAME);
  public static QName GEOMETRY_ELEMENT_NAME = WPS_PD.qname(GEOMETRY_LOCAL_NAME);
  public static QName GROUP_ELEMENT_NAME = WPS_PD.qname(VIEWPORT_LOCAL_NAME);

  public final static DataType<Link> LINK_DATA_TYPE = WPS_PD.complexDataType(
      LINK_LOCAL_NAME,
      Link.class);

  public final static DataType<Message> MESSAGE_DATA_TYPE = WPS_PD.complexDataType(
      MESSAGE_LOCAL_NAME,
      Message.class);

  public final static DataType<Viewport> VIEWPORT_DATA_TYPE = WPS_PD.complexDataType(
      VIEWPORT_LOCAL_NAME,
      Viewport.class);

  public final static DataType<Marker> MARKER_DATA_TYPE = WPS_PD.complexDataType(
      MARKER_LOCAL_NAME,
      Marker.class);

  public final static DataType<Geometry> GEOMETRY_DATA_TYPE = WPS_PD.complexDataType(
      GEOMETRY_LOCAL_NAME,
      Geometry.class);

  public final static DataType<Group> GROUP_DATA_TYPE = WPS_PD.complexDataType(
      GROUP_LOCAL_NAME,
      Group.class);
}