/**
 * Legato is a configurable, lightweight web mapping client that can be
 * easily embedded into web pages and portals, CMS and individual web
 * applications. Legato is implemented in JavaScript and based on the
 * popular open source library OpenLayers.
 *
 * Copyright (C) 2010  disy Informationssysteme GmbH, http://www.disy.net
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package net.disy.ogc.wpspd.v_1_0_0.sample;

import net.disy.ogc.wps.v_1_0_0.sample.ParameterlessAnnotatedObject;
import net.disy.ogc.wpspd.v_1_0_0.Geometry;
import net.disy.ogc.wpspd.v_1_0_0.Marker;
import net.disy.ogc.wpspd.v_1_0_0.MarkerType;
import net.disy.ogc.wpspd.v_1_0_0.Message;
import net.disy.ogc.wpspd.v_1_0_0.ObjectFactory;
import net.disy.ogc.wpspd.v_1_0_0.WpspdUtils;

public abstract class AbstractSampleMarkerAnnotatedObject
    implements
    ParameterlessAnnotatedObject<Marker> {

  private final ObjectFactory objectFactory = WpspdUtils.createObjectFactory();
  private final ParameterlessAnnotatedObject<Geometry> geometryAnnotatedObject;
  private final ParameterlessAnnotatedObject<Message> messageAnnotatedObject;

  public AbstractSampleMarkerAnnotatedObject(
      ParameterlessAnnotatedObject<Geometry> geometryAnnotatedObject,
      ParameterlessAnnotatedObject<Message> messageAnnotatedObject) {
    this.geometryAnnotatedObject = geometryAnnotatedObject;
    this.messageAnnotatedObject = messageAnnotatedObject;
  }

  public Marker execute() {
    final MarkerType markerType = objectFactory.createMarkerType();
    final Marker marker = objectFactory.createMarker(markerType);

    if (geometryAnnotatedObject != null) {
      markerType.setGeometry(geometryAnnotatedObject.execute());
    }
    if (messageAnnotatedObject != null) {
      markerType.setMessage(messageAnnotatedObject.execute());
    }
    return marker;
  }

}
